

model InductorDummy "Ideal linear electrical inductor" 
  extends Modelica.Electrical.Analog.Interfaces.OnePort;
  parameter Modelica.SIunits.Inductance L=1 "Inductance";
  Real di;
  annotation (
    Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]),
    Documentation(info="<HTML>
<P>
The linear inductor connects the branch voltage <i>v</i> with the
branch current <i>i</i> by  <i>v = L * di/dt</i>.
The Inductance <i>L</i> is allowed to be positive, zero, or negative.
</p>
</HTML>
"), Icon(
      Ellipse(extent=[-60, -15; -30, 15]),
      Ellipse(extent=[-30, -15; 0, 15]),
      Ellipse(extent=[0, -15; 30, 15]),
      Ellipse(extent=[30, -15; 60, 15]),
      Rectangle(extent=[-60, -30; 60, 0], style(color=7, fillColor=7)),
      Line(points=[60, 0; 90, 0]),
      Line(points=[-90, 0; -60, 0]),
      Text(
        extent=[-100, -60; 100, -120],
        string="L=%L",
        style(color=0)),
      Text(extent=[-136,40; 148,100],   string="%name")),
    Diagram(
      Ellipse(extent=[-60, -15; -30, 15]),
      Ellipse(extent=[-30, -15; 0, 15]),
      Ellipse(extent=[0, -15; 30, 15]),
      Ellipse(extent=[30, -15; 60, 15]),
      Rectangle(extent=[-60, -30; 60, 0], style(color=7, fillColor=7)),
      Line(points=[60, 0; 90, 0]),
      Line(points=[-90, 0; -60, 0])),
    Window(
      x=0.3,
      y=0.12,
      width=0.6,
      height=0.6),
    uses(Modelica(version="1.6")));
equation 
  L*di = v;
end InductorDummy;
