package BaseClasses 
  model PartialFourWheeler "Bicycle model of a car chassis" 
    parameter Modelica.SIunits.Mass m=1200 "|Mass and Inertia|Mass of body";
    parameter Modelica.SIunits.Inertia I=1800 "|Mass and Inertia|Inertia";
    parameter Modelica.SIunits.Distance f=1.0 
      "|Geometry|Distance from COG to front wheels";
    parameter Modelica.SIunits.Distance b=1.7 
      "|Geometry|Distance from COG to rear wheels";
    parameter Modelica.SIunits.Distance tw=1.7 "|Geometry|track width";
    
     parameter Modelica.SIunits.Position[2] r_1={f,tw/2} 
      "|Geometry|Advanced|Position of wheel 1 relative to COG";
     parameter Modelica.SIunits.Position[2] r_2={f,-tw/2} 
      "|Geometry|Advanced|Position of wheel 2 relative to COG";
     parameter Modelica.SIunits.Position[2] r_3={-b,tw/2} 
      "|Geometry|Advanced|Position of wheel 3 relative to COG";
     parameter Modelica.SIunits.Position[2] r_4={-b,-tw/2} 
      "|Geometry|Advanced|Position of wheel 4 relative to COG";
    
    parameter Real vx_init=20 "|Initial conditions|initial speed";
    
    parameter Real x0=0 "|Initial conditions|initial position x";
    parameter Real y0=0 "|Initial conditions|initial posiiton y";
    
    parameter Real iSW=1/15 "|Geometry|Steering wheel gain";
    
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[2, 2],
        component=[20, 20]),
      Diagram,
      Icon,
      Window(
        x=0.1,
        y=0.01,
        width=0.63,
        height=0.81),
      Documentation(info="<HTML><p>
This model is a simplification of a car chassis. The roll, pitch and bounce motions are neglected and no load transfer between the wheels can occur. The front and rear axis' wheels are replaced by a single wheel each, leaving a model that looks like a bicycle.
</p></HTML>
"));
    
    PlanarMultiBody.Parts.InertialSystem ground 
                                annotation (extent=[-20,-60; 0,-40]);
    PlanarMultiBody.Joints.Revolute steering_2 
      annotation (extent=[-50,20; -70,40], rotation=90);
    
    PlanarMultiBody.Examples.PlanarVehicle.SimpleWheel wheel_1(
        iWheel(w(start=vx_init/wheel_1.radius))) 
                         annotation (extent=[-76,-76; -44,-46],
                                                             rotation=270);
    PlanarMultiBody.Examples.PlanarVehicle.SimpleWheel wheel_2(
        iWheel(w(start=vx_init/wheel_2.radius)), leftWheel=false) 
                                               annotation (extent=[-76,46; -44,76],rotation=90);
    PlanarMultiBody.Examples.PlanarVehicle.SimpleWheel wheel_3(
        iWheel(w(start=vx_init/wheel_3.radius))) 
      annotation (extent=[44,-76; 76,-46],  rotation=270);
    PlanarMultiBody.Examples.PlanarVehicle.SimpleWheel wheel_4(
        iWheel(w(start=vx_init/wheel_4.radius)), leftWheel=false) 
      annotation (extent=[44,46; 76,76],    rotation=90);
    
    replaceable PlanarMultiBody.Joints.FreeMotion freeMotion(
      v_rela(start={vx_init,0}),
      r_rela(start={x0,y0}),
      phi(start=0)) extends ChassisForVMA.PlanarMultiBody.Interfaces.TreeJoint 
      annotation (extent=[0,-40; 20,-20],     rotation=90);
    
    PlanarMultiBody.Parts.ShapeBody body(
      m=m,
      I=I,
      Shape="sphere",
      ra={-0.1,0,0},
      Length=0.2,
      Width=0.2,
      Height=0.2)           annotation (extent=[0,0; 20,20],   rotation=90);
    
    PlanarMultiBody.Parts.FrameTranslation to_1(r=r_1) 
      annotation (extent=[-64,-6; -56,-14],    rotation=90);
    PlanarMultiBody.Parts.FrameTranslation to_2(r=r_2) 
      annotation (extent=[-64,12; -56,4],      rotation=270);
    PlanarMultiBody.Examples.PlanarVehicle.AckermannSteering AckermannSteering1
      (
      wheelbase=f + b,
      trackwidth=tw,
      ratio=iSW) 
      annotation (extent=[-48,-2; -28,18]);
    PlanarMultiBody.Parts.FrameTranslation to_3(r=r_3) 
      annotation (extent=[56,-6; 64,-14],      rotation=90);
    PlanarMultiBody.Parts.FrameTranslation to_4(r=r_4) 
      annotation (extent=[56,12; 64,4],        rotation=270);
    PlanarMultiBody.Joints.Revolute steering_1 
      annotation (extent=[-50,-20; -70,-40],
                                           rotation=90);
    
  equation 
    connect(ground.frame_b, freeMotion.frame_a) annotation (points=[0.5,-50; 10,
          -50; 10,-40.5],                                 style(color=0));
    connect(to_1.frame_a, freeMotion.frame_b) annotation (points=[-60,-5.8; 10,
          -5.8; 10,-19.5],                          style(color=10));
    connect(steering_2.axis, AckermannSteering1.flange_b2) annotation (points=[
          -53,30; -44,30; -44,16], style(color=0, rgbcolor={0,0,0}));
    connect(steering_1.axis, AckermannSteering1.flange_b1) annotation (points=[
          -53,-30; -44,-30; -44,0], style(color=0, rgbcolor={0,0,0}));
    connect(freeMotion.frame_b, to_3.frame_a) annotation (points=[10,-19.5; 10,
          -5.8; 60,-5.8],                     style(color=0, rgbcolor={0,0,0}));
    connect(to_3.frame_b, wheel_3.carrierFrame) 
      annotation (points=[60,-14.2; 60,-49], style(color=0, rgbcolor={0,0,0}));
    connect(to_4.frame_b, wheel_4.carrierFrame) 
      annotation (points=[60,12.2; 60,49], style(color=0, rgbcolor={0,0,0}));
    connect(to_4.frame_a, to_3.frame_a) 
      annotation (points=[60,3.8; 60,-5.8], style(color=0, rgbcolor={0,0,0}));
    connect(wheel_2.carrierFrame, steering_2.frame_b) 
      annotation (points=[-60,49; -60,40.5], style(color=0, rgbcolor={0,0,0}));
    connect(steering_2.frame_a, to_2.frame_b) 
      annotation (points=[-60,19.5; -60,12.2], style(color=0, rgbcolor={0,0,0}));
    connect(to_2.frame_a, to_1.frame_a) 
      annotation (points=[-60,3.8; -60,-5.8], style(color=0, rgbcolor={0,0,0}));
    connect(to_1.frame_b, steering_1.frame_a) annotation (points=[-60,-14.2; -60,
          -19.5], style(color=0, rgbcolor={0,0,0}));
    connect(steering_1.frame_b, wheel_1.carrierFrame) 
      annotation (points=[-60,-40.5; -60,-49], style(color=0, rgbcolor={0,0,0}));
    connect(body.frame_a, freeMotion.frame_b) annotation (points=[10,-0.5; 10,
          -19.5],                               style(color=0, rgbcolor={0,0,0}));
  end PartialFourWheeler;
  
  package PlanarMultiBody "Modelica library to model 2D mechanical systems" 
    import Modelica.SIunits;
  extends Modelica.Icons.Library;
  annotation (
    version="0.9",
    versionDate="2002-12-31",
    Coordsys(
      extent=[0, 0; 442, 386],
      grid=[1, 1],
      component=[20, 20]),
    Window(
      x=0.45,
      y=0.01,
      width=0.28,
      height=0.42,
      library=1,
      autolayout=1),
    Documentation(info="
<HTML>
<p>
This library is used to model <b>planar mechanical</b> systems. The components of the PlanarMultiBody library can be combined
with the 1D-mechanical libraries Modelica.Mechanics.Rotational,
Modelica.Mechanics.Translational and the 3D-mechanical library ModelicaAdditions.MultiBody
The library is implemeted as a simplified ModelicaAdditions.MultiBody and is suitable when a three dimensional model is not neccessary or wanted.</p>
<p>
A first example to model a simple pendulum is given in the next figure:
</p>
<IMG SRC=\"../Images/Pendulum.png\" ALT=\"Pendulum1\">
<p>
This system is built up by the following components:
</p>
<ul>
<li> Component <b>inertial</b> is an instance of
     PlanarMultiBody.Parts.InertialSystem and defines the inertial
     system of the model as well as the gravity acceleration</li>
<li> Component <b>revolute</b> is an instance of
     PlanarMultiBody.Joints.Revolute and defines a revolute joint which
     is connected to the inertial system. The axis of rotation is
     perpendicular to the plane, i.e. if the plane is xy the axis of rotation would be z.
     Since this frame is rigidly attached to the inertial frame, these
     two frames are identical. As a consequence, the revolute joint
     rotates around the z-axis of the inertial system.</li>
<li> Component <b>shapeBody</b> is an instance of
     PlanarMultiBody.Parts.ShapeBody and defines a shape with mass and inertia properties. Additionally, the shape contains animation data, by default a box.</li>
<li> Component <b>damper</b> is an instance of
     Modelica.Mechanics.Rotational.Damper and defines the
     velocity dependent damping in the joint axis. It is connected
     between the driving flange <b>axis</b> of the joint and
     flange <b>bearing</b>. Both flanges are 1D mechanical connections.</li>
</ul>
<p>
The 2D-components are connected together at connectors <b>Frame_a</b>
or <b>Frame_b</b>. Both connectors define a coordinate system (a frame)
at a specific location of the component, which is fixed in the component.
The frame, i.e., the corresponding connector, is defined by the following
variables:
</p>
<pre>
  S[2,2]: Rotation matrix describing the frame with respect to the inertial
          frame, i.e. if ha is vector h resolved in frame_a and h0 is
          vector h resolved in the inertial frame, h0 = S*ha.
  r0[2] : Vector from the origin of the inertial frame to the origin
          of the frame, resolved in the inertial frame in [m] !!! (note,
          that all other vector quantities are resolved in frame_a!!!).
  v[2]  : Absolute (translational) velocity of the frame, resolved in
          the frame in [m/s]:  v = transpose(S)*der(r0)
  w     : Absolute angular velocity of the frame, resolved in the frame,
          in [rad/s]:  w = der(phi), where phi is calculated from S.
  a[2]  : Absolute translational acceleration of the frame minus gravity
          acceleration, resolved in the frame, in [m/s^2]:
             a = transpose(S)*( der(S*v) - ng*g )
          (ng,g are defined in model PlanarMultiBody.Parts.InertialSystem).
  z     : Absolute angular acceleration of the frame, resolved in the
          frame, in [rad/s^2]:  z = transpose(S)*der(S*w)
  f[2]  : Resultant cut-force acting at the origin of the frame,
          resolved in the frame, in [N].
  t     : Resultant cut-torque with respect to the origin of the frame,
          resolved in the frame, in [Nm].
</pre>
<p>
If two frame-connectors are connected together, the corresponding frames
are rigidly fixed together, i.e., they are identical. Usually, all vectors
of a component are expressed in <b>frame_a</b> of this component, i.e., in
a coordinate system fixed in the component. For example, the vector from
the origin of frame_a to the center-of-mass in component
PlanarMultiBody.Parts.Body is resolved in frame_a.
</p>
<p>
A more advance example is shown in the next figure.
It is the definition of a simplified chassis model, defined in PlanarMultiBody.Examples.BicycleModel.CarWithTrailer.
Roll, pitch and bounce are neglected and the front and rear axes' wheels are replaced with one single wheel each. 
This is often reffered to as the bicycle model. In this case it is also equipped with a trailer.
The external flanges allows driving and braking at each wheels and the attachment of other models to the chassis, 
such as aerodynamic models, trailers etc. The inport requires a steering signal from a driver.
</p>
<IMG SRC=\"../Images/bicycle1.png\" ALT=\"Bicycle (diagram layer)\">
<p>
After processing the model feed with a steering signal in a Modelica translator and
simulating it, an animation can be performed:
</p>
<IMG SRC=\"../Images/bicycle2.png\" ALT=\"Bicycle (animation)\">
<p>
It is also possible to define multibody systems which have kinematic
loops. An example is given in the next two figures (as object diagram
and as animation view) where a mechanism with two coupled loops and
one degree of freedom is shown:
</p>
<IMG SRC=\"../Images/loop1.png\" ALT=\"TwoLoops1\">
<p></p>
<IMG SRC=\"../Images/loop2.png\" ALT=\"TwoLoops2\">
<p>
The PlanarMultiBody library consists of the following elements:
</p>
<ul>
<li><b>Inertial system</b> (in package PlanarMultiBody.Parts):<br>
    Exactly one inertial system must be present.</li>
<li><b>Rigid bodies</b> in package PlanarMultiBody.Parts (grey icons or
  brown icons if animation information included):<br>
  All body objects have at most 2 frames where the body can be connected with
  other elements. If a rigid body has several attachment points where
  additional elements can be connected, it has to be built up by several
  body or (massless) frame elements (FrameTranslation, FrameRotationm ...)
  which are rigidly connected together.
  Presently, elastic bodies are not supported.</li>
<li><b>Joints</b> in the <b>spanning tree</b> in package PlanarMultiBody.Joints:<br>
  A general multibody system with closed kinematic loops is handeled by dividing
  the joints into two distinct sets: <b>Tree-Joints</b> and <b>Cut-Joints</b>.
  After removal of all of the Cut-Joints, the resulting system must have a
  tree-structure. All joints in subpackage <b>Joints</b>, are joints used
  in this spanning tree. The relative motion between the two cut-frames of a
  Joint is described by 0-3 generalized minimal-coordinates q
  and their first and second derivatives qd, qdd. By default, q and qd are
  used as state variables. In a kinematic loop, 3-nc degrees-of-freedom
  have to be removed, when the cut-joints introduces nc constraints.
  The one-degree-of-freedom joints (Revolute, Prismatic, Screw) may have a <b>variable</b> structure. That is, the joint can
  be <b>locked</b> and <b>unlocked</b> during the movement of a multibody system.
  This feature can be used to model brakes, clutches, stops or sticking friction.
  Locking is modelled with elements of the Modelica.Mechanics.Rotational library,
  such as classes Clutch or Friction, which can be attached to flange <b>axis</b>
  of the joints.</li>
<li><b>Cut-Joints</b> in package PlanarMultiBody.CutJoints:<br>
  All  red  joints are cut joints. Cut joints are used to
  break closed kinematic loops (see previous paragraph).</li>
<li><b>Force</b> elements in package PlanarMultiBody.Forces:<br>
  Force elements, such as springs and dampers, can be attached between
  two points of distinct bodies or joints. However, it is <b>not possible
  to connect force elements with other force elements</b>. It is easy for an
  user to introduce new force elements as subclasses from already existing
  ones (e.g. from model class PlanarMultiBody.Interfaces.LineForce).
  One-dimensional force laws can be
  used from the Mechanics.Rotational library.
  Gravitational forces for <b>all</b> bodies are taken into account by setting
  the gravitational acceleration of the inertial system (= object of
  PlanarMultiBody.Parts.InertialSystem) to a nonzero value.</li>
<li><b>Sensor</b> elements in package PlanarMultiBody.Sensors (yellow icons):<br>
  Between two distinct points of bodies and joints a sensor element can be
  attached. A sensor is used to calculate relative kinematic quantities
  between the two points. In the libraries a general 3D sensor element
  (calculate all relative quantities) and a line-sensor element are
  present. </li>
</ul>
<p>
<b>Connection Rules</b>:<br>
The elements of the multibody library cannot be connected arbitrarily
together. Instead the following rules hold:
</p>
<ol>
<li><b>Tree joint</b> objects, <b>body</b> objects
    and the <b>inertial system</b> have to be connected together in such a way
    that a <b>frame_a</b> of an object (cut filled with blue color) is always connected
    to a <b>frame_b</b> of an object (non-filled cut). The connection structure
    has to form a <b>tree</b> with the inertial system as a root.</li>
<li><b>Cut-joint</b>, <b>force</b>, and <b>sensor</b> objects have to be
    always connected
    between two frames of a <b>tree joint</b>, <b>body</b> or
    <b>inertial system</b> object.
    E.g., it is not allowed to connect two force objects together.
    </li>
<li>By using the <b>input/output</b> prefixes of Modelica in the corresponding
    connectors of the PlanarMultiBody library, it is guaranteed that
    only connections can be carried out, for which the library is
    designed.</li>
</ol>
<p>
Note, this library utilizes the non-standard function <b>constrain(..)</b>
and assumes that this function is supported by the Modelica translator:
</p>
<pre>
   Real r[:], rd[:], rdd[:];
      ...
   r   = ..
   rd  = ...
   rdd = ...
   constrain(r,rd,rdd);
</pre>
<p>
where r, rd and rdd are variables which need to be computed
somewhere else. Function constrain()
is used to explicitly inform the Modelica translator that
rd is the derivative of r and rdd is the derivative of rd
and that all derivatives need to be identical to zero.
The Modelica translator can utilize this information to use
rd and rdd whenever the Pantelides algorithm requires to compute
the derivatives of r. This enhances the efficiency considerably.
A simple, but inefficient, implementation of constrain() is:
</p>
<pre>
   r = 0;
</pre>
<p>
In the multibody library, function constrain() is used in the
cut joints, i.e., whenever kinematic loops are present.
</p>
<p>
<b>References</b>
</p>
<pre>
The library and its documentation is based on ModelicaAdditions.MultiBody by Martin Otter.</pre>
<dl>
<dt><b>Main Author:</b>
<dd><a href=\"http://www.ave.kth.se\">Johan Andreasson</a><br>
    KTH Vehicle Dynamics<br>
    Teknikringen 8<br>
    10044 Stockholm<br>
    Sweden<br>
    email: <A HREF=\"mailto:johan@fkt.kht.se\">johan@fkt.kth.se</A><br>
</dl>
<p><b>Copyright (C) 2002, Johan Andreasson.</b></p>
<p><i>
The PlanarMultiBody package is <b>free</b> software;
it can be redistributed and/or modified
under the terms of the <b>Modelica license</b>, see the license conditions
and the accompanying <b>disclaimer</b> in the documentation of package
Modelica in file \"Modelica/package.mo\".
</i></p>
</HTML>
"), uses(Modelica(version="1.6"), ModelicaAdditions(version="1.5")));
    
  package Examples "Package with examples" 
    extends Modelica.Icons.Library;
    annotation (
      Coordsys(
        extent=[0, 0; 442, 386],
        grid=[2, 2],
        component=[20, 20]),
      Window(
        x=0.6,
        y=0.17,
        width=0.28,
        height=0.42,
        library=1,
        autolayout=1),
      Documentation(info="<HTML>
<p>This package contains examples to illustrate the use of PlanarMultiBody.
</p>
</HTML>"));
      
    package PlanarVehicle "Biclycle models of a car" 
      extends Modelica.Icons.Example;
      annotation (
        Coordsys(
          extent=[0, 0; 440, 450],
          grid=[2, 2],
          component=[20, 20]),
        Window(
          x=0.45,
          y=0.01,
          width=0.28,
          height=0.42,
          library=1,
          autolayout=1),
        Documentation(info="<HTML>
<p> This package shows some examples relating to vehicles based on a widely used approximation technique to only study one plane of the vehicle motion. 
The planes that are used is typically:
</p>
<ul>
<li> Symmetry plane: Bounce and pitch are used to study comfort and behaviour during braking/acceleration.
</li>
<li> Assymetry plane: Yaw, lateral and longitudinal motions are used to study handling.
</li>
<li> Roll plane: Roll and bounce motions are used to study the influence of different suspension characteristics.
</li>
</ul>
<p>
The examples here consider motion within the assymetry plane based on a bicycle model. This model were derived to 
catch the essentials of a car's dynamics. Roll, pitch and bounce are neglected and the wheelpairs front and rear are repleced by a single wheel each.
The package contains two complete models; BicycleModel and BicycleWithTrailer. 
</p>
</HTML>
"));
        
      model SimpleWheel 
          import VisualVector;
        parameter Boolean leftWheel=true;
        parameter Real radius=0.3;
        parameter Real Fz_nom=3000 "Nominal normal force";
        parameter Real camberAngle=0;
        parameter Real I=1;
          
        parameter Real Fds_x=100000 "Slope at s_x=0 in longitudinal direction";
        parameter Real s_max_x=0.15 "Slip of maximum longitudinal tyre force";
        parameter Real F_max_x=6000 "maximum longitudinal tyre force";
        parameter Real s_slide_x=0.4 "Slip of sliding begin";
        parameter Real F_slide_x=5500 
            "Longitudinal tyre force at sliding begin";
          
        parameter Real Fds_y=80000 "Slope at s_y=0 in lateral direction";
        parameter Real s_max_y=0.21 "Slip of maximum lateral tyre force";
        parameter Real F_max_y=5500 "maximum lateral tyre force";
        parameter Real s_slide_y=0.6 "Slip of sliding begin";
        parameter Real F_slide_y=5000 "Lateral tyre force at sliding begin";
          
        parameter Real dL0=0.133 
            "(distance contact point to f_y) / (length of contact area) for zero lateral slip";
        parameter Real s0=0.25 
            "Lateral slip where dL=0 for the first time (usually, a bit higher as s_Fmax_y)";
        parameter Real sE=0.6 
            "Lateral slip where dL remains zero for higher slips";
          
        Real w;
          
        parameter Real v_eps=0.000001;
          
        Real slip;
        Real sinphi;
        Real cosphi;
        Real v_abs;
        Real Rw;
          
        Real Fx;
        Real Fy;
        Real dL;
        Real F;
        Real df_ds;
          
        Real Fds;
        Real F_max;
        Real F_slide;
        Real s_slide;
        Real s_max;
        Real sigma;
        Real s_y;
        Real c;
          
        Real va[2];
        protected 
        parameter Real left=if leftWheel then 1 else -1;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Polygon(points=[-44, 78; -44, 74; -56, 52; -54, 46; -42, 26; -42, 16;
                   -52, -6; -56, -12; -56, -26; -54, -32; -46, -40; -36, -44; -26,
                   -46; -12, -50; -2, -56; 14, -64; 20, -66; 44, -70; 58, -68; 64,
                   -64; 68, -54; 68, -46; 64, -22; 60, -10; 60, 10; 72, 16; 78,
                  24; 84, 30; 92, 38; 94, 46; 94, 56; 88, 64; 82, 70; 78, 72; 64,
                   74; 42, 78; 8, 84; -4, 90; -4, 94; -10, 98; -16, 96; -24, 92;
                  -26, 90; -30, 82; -32, 80; -36, 80; -44, 78], style(color=8,
                  fillColor=8)),
            Rectangle(extent=[40, 10; 72, -10], style(
                color=0,
                gradient=2,
                fillColor=8)),
            Rectangle(extent=[-48, 90; 50, -90], style(gradient=2, fillColor=10)),
            Rectangle(extent=[-70, 10; -48, -10], style(color=9, fillColor=9)),
            Line(points=[-4, 0; -4, -76], style(color=7)),
            Line(points=[-4, 0; 38, 0], style(color=7)),
            Text(
              extent=[20, 36; 46, 6],
              string="y",
              style(color=7)),
            Text(
              extent=[0, -60; 26, -90],
              string="x",
              style(color=7)),
            Polygon(points=[38, 4; 38, -4; 48, 0; 38, 4], style(color=7,
                  fillColor=7)),
            Polygon(points=[-6, -76; 0, -76; -4, -86; -8, -76; -6, -76], style(
                  color=7, fillColor=7)),
            Polygon(points=[-44, 78; -44, 74; -56, 52; -54, 46; -42, 26; -42, 16;
                   -52, -6; -56, -12; -56, -26; -54, -32; -46, -40; -36, -44; -26,
                   -46; -12, -50; -2, -56; 14, -64; 20, -66; 44, -70; 58, -68; 64,
                   -64; 68, -54; 68, -46; 64, -22; 60, -10; 60, 10; 72, 16; 78,
                  24; 84, 30; 92, 38; 94, 46; 94, 56; 88, 64; 82, 70; 78, 72; 64,
                   74; 42, 78; 8, 84; -4, 90; -4, 94; -10, 98; -16, 96; -24, 92;
                  -26, 90; -30, 82; -32, 80; -36, 80; -44, 78], style(color=8,
                  fillColor=8)),
            Rectangle(extent=[40, 10; 60, -10], style(
                color=0,
                gradient=2,
                fillColor=8)),
            Rectangle(extent=[-48, 90; 50, -90], style(gradient=2, fillColor=10)),
            Rectangle(extent=[-60, 10; -48, -10], style(color=9, fillColor=9)),
            Line(points=[-4, 0; -4, -76], style(color=7)),
            Line(points=[-4, 0; 38, 0], style(color=7)),
            Text(
              extent=[20, 36; 46, 6],
              string="y",
              style(color=7)),
            Text(
              extent=[0, -60; 26, -90],
              string="x",
              style(color=7)),
            Polygon(points=[38, 4; 38, -4; 48, 0; 38, 4], style(color=7,
                  fillColor=7)),
            Polygon(points=[-6, -76; 0, -76; -4, -86; -8, -76; -6, -76], style(
                  color=7, fillColor=7))),
          Diagram,
          Window(
            x=0.02,
            y=0.01,
            width=0.59,
            height=0.66),
          Documentation(info="<HTML><p> A simple model of a wheel, taking account of interaction between longitudinal an lateral forces and tyre force saturation. The tyre characteristics can be modified with paramters for side stiffnes, maximum force, force when sliding and at what slip these occur,
see more information at each parmeter. 
</p></HTML>
"));
          
        public 
        Interfaces.Frame_a carrierFrame 
          annotation (extent=[-90, -10; -70, 10], layer="icon");
        Modelica.Mechanics.Rotational.Interfaces.Flange_a driveShaft annotation (
          extent=[92, 10; 72, -10],
          rotation=180,
          layer="icon");
        Modelica.Mechanics.Rotational.Inertia iWheel(J=I) 
          annotation (extent=[30, 0; 50, 20], rotation=180);
        Modelica.Mechanics.Rotational.Torque driveTorque 
          annotation (extent=[30, -30; 50, -10]);
        Modelica.Mechanics.Rotational.Sensors.SpeedSensor wheelSpeed 
          annotation (extent=[30, -60; 50, -40], rotation=180);
        Parts.Shape wheel(
          r0={0,0,radius},
          Shape="pipe",
          LengthDirection={0,left,0},
          WidthDirection={1,0,0},
          Length=0.2,
          Width=radius*2,
          Height=radius*2,
          Material={0.5,0.5,0.5,0.5},
          Extra=0.75) annotation (extent=[-8, 26; 32, 66]);
        Forces.ExtForce force 
          annotation (extent=[-40, -60; -20, -40], rotation=180);
        Forces.ExtTorque torque 
          annotation (extent=[-40, -40; -20, -20], rotation=180);
        VisualVector X(
          category="force",
          r={carrierFrame.r0[1],carrierFrame.r0[2],0},
          Size={-carrierFrame.f[1],0,0},
          S=[carrierFrame.S[1, 1], carrierFrame.S[1, 2], 0; carrierFrame.S[2, 1],
               carrierFrame.S[2, 2], 0; 0, 0, 1]) 
          annotation (extent=[-70, 90; -50, 70]);
        VisualVector Y(
          Material={0,1,0,0.5},
          category="force",
          r={carrierFrame.r0[1],carrierFrame.r0[2],0},
          Size={0,-carrierFrame.f[2],0},
          S=[carrierFrame.S[1, 1], carrierFrame.S[1, 2], 0; carrierFrame.S[2, 1],
               carrierFrame.S[2, 2], 0; 0, 0, 1]) 
          annotation (extent=[-30, 90; -10, 70]);
      equation 
        connect(iWheel.flange_a, driveShaft) 
          annotation (points=[50,10; 62,10; 62,0; 82,0],     style(color=0));
        connect(wheelSpeed.flange_a, driveShaft) 
          annotation (points=[50, -50; 82, -50; 82, 0], style(color=0));
        connect(driveTorque.flange_b, driveShaft) 
          annotation (points=[50, -20; 64, -20; 64, 0; 82, 0], style(color=0));
        connect(wheel.frame_a, carrierFrame) annotation (points=[-9, 45.8; -45,
              45.8; -45, 0; -80, 0], style(color=10));
        connect(force.frame_b, carrierFrame) annotation (points=[-40.5, -49.9; -80,
               -49.9; -80, 0], style(color=10));
        connect(torque.frame_b, carrierFrame) annotation (points=[-40.5, -29.9; -59,
               -29.9; -59, 0; -80, 0], style(color=10));
          
          driveTorque.tau            = -radius*(Fx);
        w =wheelSpeed.w;
          
        //driveShaft.tau = radius*(Fx);
        //w = iWheel.w;//der(driveShaft.phi);
        va = carrierFrame.v;
          
        /*
       Determine slip
       v_abs: Absolute value of contact point velocity (= sqrt(Wv*Wv))
  
       phi   : Angle between longitudinal and lateral contact point velocity
  
  
                 Assumption: For Wv_abs small -> Wv_x = Wv_abs, Wv_y = 0, i.e.,
                           velocity only in longitudinal direction.
       sinphi: sin(phi) (= 0 for Wv_abs = 0)
       cosphi: cos(phi) (= -1 for Wv_abs = 0)
  */
          
        v_abs = noEvent(max([sqrt((radius*w - va[1])*(radius*w - va[1]) + va[2]*
          va[2]); 1.e-10]));
        Rw = radius*abs(w);
        slip = v_abs/(Rw + 0.1);
          
        if noEvent(v_abs > v_eps) then
          sinphi = -va[2]/v_abs*left;
          cosphi = (radius*w - va[1])/v_abs*left;
        else
          sinphi = 0;
          cosphi = -1;
        end if;
          
        Fds = noEvent(max([sqrt((Fds_x*cosphi)^2 + (Fds_y*sinphi)^2); 1.e-10]));
        s_max = noEvent(max([sqrt((s_max_x*cosphi)^2 + (s_max_y*sinphi)^2);
          1.e-10]));
        F_max = noEvent(max([sqrt((F_max_x*cosphi)^2 + (F_max_y*sinphi)^2);
          1.e-10]));
        s_slide = noEvent(max([sqrt((s_slide_x*cosphi)^2 + (s_slide_y*sinphi)^2);
           1.0001*s_max]));
        F_slide = noEvent(max([sqrt((F_slide_x*cosphi)^2 + (F_slide_y*sinphi)^2);
           1.e-10]));
          
        // Compute tyre force in contact plane
        if noEvent(slip <= s_max) then
          sigma = slip/s_max;
          c = noEvent(sigma*(s_max/max([F_max, 1.e-10])*Fds - 2 + sigma) + 1);
          F = s_max*Fds*sigma/c*left;
          df_ds = Fds*(1 - sigma)/(c*c);
        elseif noEvent(slip <= s_slide) then
          sigma = (slip - s_max)/(s_slide - s_max);
          c = 0;
          F = (F_max - (F_max - F_slide)*sigma*sigma*(3 - 2*sigma))*left;
          df_ds = -(F_max - F_slide)*6*sigma*(1 - sigma)/(s_slide - s_max);
        else
          sigma = 0;
          c = 0;
          F = (F_slide)*left;
          df_ds = 0;
        end if;
          
        // Compute attachment point of f_y with respect to contact length (dL)
        s_y = slip*sinphi;
        dL = noEvent(if abs(s_y) <= s0 then dL0*(1 - abs(s_y)/s0) else if abs(s_y)
           <= sE then -dL0*(abs(s_y) - s0)/s0*((sE - abs(s_y))/(sE - s0))^2 else 
          0);
          
        // Compute longitudinal and lateral tire forces
        Fx = F*cosphi;
        Fy = F*sinphi;
        // + Fz_nom*tan(camberAngle);
          force.inPort      = {Fx,Fy};
          torque.inPort         = -dL*Fy;
          
      end SimpleWheel;
        
      model AckermannSteering 
          "AckermannSteering - steers the right and left wheel according to the Ackermann geometry." 
          
        parameter Real wheelbase=2.55 "Distance from front to rear axis";
        parameter Real trackwidth=1.6 "Distance between the front wheels";
        parameter Real ratio=1 
            "Gear ratio to be used to compensate for the ratio in a steering linkage, should be angle on steering wheel/steer angle";
        protected 
        parameter Real b=wheelbase;
        parameter Real tw=trackwidth;
          
        Real tand;
        Real delta;
        Real delta_1;
        Real delta_2;
        Real ratio_1;
        Real ratio_2;
        //  Real w_1;
        //  Real w_2;
        //  Real w;
          
        //SI.Position r1,r1d,r2,r2d;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[62, 40; 72, -40], style(gradient=2, fillColor=8)),
            Rectangle(extent=[-80, 60; 40, -60], style(color=0, fillColor=51)),
            Rectangle(extent=[4, -6; 22, -14], style(color=10, fillColor=10)),
            Rectangle(extent=[4, 44; 22, 36], style(color=10, fillColor=10)),
            Line(points=[12, -52; -56, 40]),
            Polygon(points=[-62, 32; -48, 42; -52, 48; -66, 38; -62, 32], style(
                  color=10, fillColor=10)),
            Line(points=[12, -52; -56, -10]),
            Polygon(points=[-64, -14; -54, 0; -48, -4; -58, -18; -64, -14], style(
                  color=10, fillColor=10)),
            Rectangle(extent=[40, 4; 62, -4], style(gradient=2, fillColor=8)),
            Text(extent=[-76, -22; -44, -58], string="L")),
          Window(
            x=0.45,
            y=0.01,
            width=0.55,
            height=0.95),
          Diagram(Line(points=[-50, -80; 0, -86; 22, -40; 22, 20; 2, 66; -50, 80])),
          Documentation(info="<HTML>
<p>
A geometrical approach shows that the outer wheel should turn less when the car turns.
The inner wheel should turn with an angle delta_i=atan(L/R) while the outer wheel should turn with a smaller angle delta_o=atan(L/(b+R)) where R is the turning radius of the inner wheel, b is the trackwidth and L is the wheelbase.
This effect is often reffered to as Ackermann steering but was devided by Lankensperger.
This model assumes that the driver steers a wheel in the midle of the car and from this, the inner and outer turning angles are calculated.
</p>
</HTML>
"));
          
        public 
        Modelica.Mechanics.Rotational.Interfaces.Flange_a flange_a 
          annotation (extent=[80, -16; 110, 16]);
        Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b1 annotation (
          extent=[-50, -90; -70, -70],
          rotation=90,
          layer="icon");
        Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b2 annotation (
          extent=[-50, 70; -70, 90],
          rotation=90,
          layer="icon");
      equation 
        // geometric equations for steering angles
        flange_b1.phi = delta_1;
        flange_b2.phi = delta_2;
        flange_a.phi = delta;
          
        tand = tan(ratio*delta);
        //delta_1 = atan(2*wheelbase*tand/(2*wheelbase - trackwidth*tand));
        //delta_2 = atan(2*wheelbase*tand/(2*wheelbase + trackwidth*tand));
          
        //  w_1 = der(delta_1);
        //  w_2 = der(delta_2);
        //  w = der(delta);
          
        //  w_1 = ratio_1*w;
        //  w_2 = ratio_2*w;
          
        delta_1 = (ratio*ratio_1)*delta;
        delta_2 = (ratio*ratio_2)*delta;
          
        // Torque equations  
        0 = flange_a.tau + ratio*ratio_1*flange_b1.tau + ratio*ratio_2*flange_b2.
          tau;
          
        // Derivatives delta_1/delta and delta_2/delta
        ratio_1 = 4*b*b*(1 + tand*tand)/(4*b^2 - 4*b*tw*tand + tw*tw*tand^2 + 4*b
          *b*tand^2);
        ratio_2 = 4*b*b*(1 + tand*tand)/(4*b^2 + 4*b*tw*tand + tw*tw*tand^2 + 4*b
          *b*tand^2);
          
      end AckermannSteering;
    end PlanarVehicle;
  end Examples;
    
    package Forces "Force and torque elements for 2D mechanical components" 
      extends Modelica.Icons.Library;
      annotation (
        Coordsys(
          extent=[0, 0; 442, 386],
          grid=[1, 1],
          component=[20, 20]),
        Window(
          x=0,
          y=0.6,
          width=0.4,
          height=0.4,
          library=1,
          autolayout=1),
        Documentation(info="
<HTML>
<p>
This package contains components to model forces and torques.
</p>
</HTML>"));
      
      model ExtForce "External force" 
        annotation (
          preferedView="info",
          defaultComponentName="frc",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Polygon(points=[-80, 8; 30, 8; 30, 30; 80, 0; 30, -30; 30, -8; -80,
                  -8; -80, 8], style(
                color=0,
                gradient=0,
                fillColor=0,
                fillPattern=1)), Text(extent=[-100, 100; 100, 40], string="%name")),
          Diagram(Polygon(points=[-80, 8; 30, 8; 30, 30; 80, 0; 30, -30; 30, -8; -80,
                   -8; -80, 8], style(
                color=0,
                gradient=0,
                fillColor=0,
                fillPattern=1))),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42),
          Documentation(info="
An external force element exerts the inport signal as negative
force on frame_b.
"));
        
        Interfaces.Frame_a frame_b 
          annotation (extent=[90, -16; 120, 14], rotation=180);
        Modelica.Blocks.Interfaces.RealInput inPort[2] 
          annotation (extent=[-110, -10; -90, 10], layer="icon");
      equation 
        frame_b.f = -inPort;
        frame_b.t = 0;
      end ExtForce;
      
      model ExtTorque "External Torque" 
        annotation (
          preferedView="info",
          defaultComponentName="trq",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Text(extent=[-100, 114; 100, 54], string="%name"),
            Line(points=[-88, 0; -64, 30; -36, 52; -2, 62; 28, 56; 48, 44; 64, 28;
                  76, 14; 80, 10], style(
                color=0,
                thickness=2,
                fillColor=0)),
            Polygon(points=[86, 0; 66, 58; 38, 28; 86, 0], style(
                color=0,
                fillColor=0,
                fillPattern=1))),
          Diagram(Line(points=[-88, 0; -64, 30; -36, 52; -2, 62; 28, 56; 48, 44; 64,
                   28; 76, 14; 80, 10], style(
                color=0,
                thickness=2,
                fillColor=0)), Polygon(points=[86, 0; 66, 58; 38, 28; 86, 0], style(
                color=0,
                fillColor=0,
                fillPattern=1))),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42),
          Documentation(info="
An external torque element exerts the inport signals as
negative torque on frame_b.
"));
        Interfaces.Frame_a frame_b 
          annotation (extent=[90, -16; 120, 14], rotation=180);
        Modelica.Blocks.Interfaces.RealInput inPort 
          annotation (extent=[-110, -10; -90, 10], layer="icon");
      equation 
        frame_b.t = -inPort;
        frame_b.f = {0,0};
      end ExtTorque;
    end Forces;
    
    package Interfaces 
      extends Modelica.Icons.Library;
      annotation (
        Coordsys(
          extent=[0, 0; 552, 534],
          grid=[2, 2],
          component=[20, 20]),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49,
          library=1,
          autolayout=1),
        Documentation(info="<HTML>
<p>This package contains interface models.</p>
</HTML>"));
      
      connector Frame_a "Frame a of a mechanical element" 
        SIunits.Position r0[2] 
          "Position vector from inertial system to frame origin, resolved in inertial system";
        Real S[2, 2] "Transformation matrix from frame_a to inertial system";
        SIunits.Velocity v[2] 
          "Absolute velocity of frame origin, resolved in frame_a";
        SIunits.AngularVelocity w 
          "Absolute angular velocity of frame_a, resolved in frame_a";
        SIunits.Acceleration a[2] 
          "Absolute acceleration of frame origin, resolved in frame_a";
        SIunits.AngularAcceleration z 
          "Absolute angular acceleration of frame_a, resolved in frame_a";
        flow SIunits.Force f[2];
        flow SIunits.Torque t;
        
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Documentation(info="<HTML>
<p> Frame a  of a mechanical element. </p>
<p> All mechanical components are always connected together at
frames. A frame is a coordinate system in the (mechanical)
cut-plane of the connection point. The variables of the cut-plane
are defined with respect to the corresponding frame_a and have the
following meaning: </p>
<HTML>
<p> Frame b  of a mechanical element. </p>
<p> All mechanical components are always connected together at
frames. A frame is a coordinate system in the (mechanical)
cut-plane of the connection point. The variables of the cut-plane
are defined with respect to the corresponding frame_b and have the
following meaning: </p>
<pre>
  S[2,2]: Rotation matrix describing the frame with respect to the inertial
          frame, i.e. if ha is vector h resolved in frame_a and h0 is
          vector h resolved in the inertial frame, h0 = S*ha.
  r0[2] : Vector from the origin of the inertial frame to the origin
          of the frame, resolved in the inertial frame in [m] !!! (note,
          that all other vector quantities are resolved in frame_a!!!).
  v[2]  : Absolute (translational) velocity of the frame, resolved in
          the frame in [m/s]:  v = transpose(S)*der(r0)
  w     : Absolute angular velocity of the frame, resolved in the frame,
          in [rad/s]:  w = der(phi), where phi is calculated from S.
  a[2]  : Absolute translational acceleration of the frame minus gravity
          acceleration, resolved in the frame, in [m/s^2]:
             a = transpose(S)*( der(S*v) - ng*g )
          (ng,g are defined in model PlanarMultiBody.Parts.InertialSystem).
  z     : Absolute angular acceleration of the frame, resolved in the
          frame, in [rad/s^2]:  z = transpose(S)*der(S*w)
flow variables:
  f[2]  : Resultant cut-force acting at the origin of the frame,
          resolved in the frame, in [N].
  t     : Resultant cut-torque with respect to the origin of the frame,
          resolved in the frame, in [Nm].
</pre>
 <HTML> 
</HTML> 
"),       Icon(Polygon(points=[0, 0; -100, -100; 100, -100; 100, 100; -100, 100; 0,
                   0], style(color=0, fillColor=9))),
          Diagram(
            Text(
              extent=[-100, -120; 100, -220],
              string="%name",
              style(color=0)),
            Rectangle(extent=[0, 100; 100, -100], style(color=0, fillColor=0)),
            Text(
              extent=[-100, -120; 100, -220],
              string="%name",
              style(color=0)),
            Rectangle(extent=[0, -100; 100, 100], style(color=58, fillColor=58)),
            Line(points=[0, 0; -100, 100; 100, 100; 100, -100; -100, -100; 0, 0],
                style(color=0)),
            Polygon(points=[0, 0; -100, -100; 100, -100; 100, 100; -100, 100; 0, 0],
                 style(color=0, fillColor=9))),
          Terminal(Polygon(points=[0, 0; -100, -100; 100, -100; 100, 100; -100, 100;
                   0, 0], style(fillColor=3))),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42));
        
      end Frame_a;
      
      connector Frame_b "Frame b of a mechanical element" 
        output SIunits.Position r0[2] 
          "Position vector from inertial system to frame origin, resolved in inertial system";
        Real S[2, 2] "Transformation matrix from frame_a to inertial system";
        SIunits.Velocity v[2] 
          "Absolute velocity of frame origin, resolved in frame_a";
        SIunits.AngularVelocity w 
          "Absolute angular velocity of frame_a, resolved in frame_a";
        SIunits.Acceleration a[2] 
          "Absolute acceleration of frame origin, resolved in frame_a";
        SIunits.AngularAcceleration z 
          "Absolute angular acceleration of frame_a, resolved in frame_a";
        flow SIunits.Force f[2];
        flow SIunits.Torque t;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Documentation(info="<HTML>
<p> Frame b  of a mechanical element. </p>
<p> All mechanical components are always connected together at
frames. A frame is a coordinate system in the (mechanical)
cut-plane of the connection point. The variables of the cut-plane
are defined with respect to the corresponding frame_b and have the
following meaning: </p>
<pre> Potential variables:
  S : Rotation matrix describing frame_b with respect to the inertial
      frame, i.e. if ha is vector h resolved in frame_b and h0 is
      vector h resolved in the inertial frame, h0 = S*ha.
  r0: Vector from the origin of the inertial frame to the origin
      of frame_a, resolved in the inertial frame in [m] !!! (note,
      that all other vector quantities are resolved in frame_a!!!).
  v : Absolute (translational) velocity of frame_a, resolved in a,
      in [m/s]:  v = transpose(S)*der(r0)
  w : Absolute angular velocity of frame_a, resolved in a,
      in [rad/s]  :  w = vec(transpose(S)*der(S));  Note, that
                   |   0 -w3  w2 |
         skew(w) = |  w3   0 -w1 | and w=vec(skew(w))
                   | -w2  w1   0 |
  a : Absolute translational acceleration of frame_b - gravity
      acceleration, resolved in a, in [m/s^2]:
          a = transpose(S)*( der(S*v) - ng*g )
      (ng,g are defined in model PlanarMultiBody.Parts.InertialSystem).
  z : Absolute angular acceleration of frame_a, resolved in a,
      in [rad/s^2]:  z = transpose(S)*der(S*w)
Flow variables:
  f : Resultant cut-force acting at the origin of frame_a,
      resolved in a, in [N].
  t : Resultant cut-torque with respect to the origin of frame_a,
      resolved in a, in [Nm].
</pre> <HTML> "),
          Icon(Polygon(points=[-100, -100; 0, -100; 100, 0; 0, 100; -100, 100; -100,
                   -100], style(color=0, fillColor=7))),
          Diagram(Text(
              extent=[-100, -120; 100, -220],
              string="%name",
              style(color=0)), Polygon(points=[-100, -100; 0, -100; 100, 0; 0, 100;
                   -100, 100; -100, -100], style(color=0, fillColor=7))),
          Terminal(Polygon(points=[-100, -100; 0, -100; 100, 0; 0, 100; -100, 100;
                  -100, -100])),
          Window(
            x=0.45,
            y=0.01,
            width=0.35,
            height=0.49));
        
      end Frame_b;
      
      partial model OneFrame_a 
        "Superclass of elements with ONE mechanical frame_a" 
        
      protected 
        Real Sa[2, 2](start=identity(2));
        SIunits.Position r0a[2];
        SIunits.Velocity va[2];
        SIunits.AngularVelocity wa;
        SIunits.Acceleration aa[2];
        SIunits.AngularAcceleration za;
        SIunits.Force fa[2];
        SIunits.Torque ta;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42),
          Documentation(info="p>
Superclass of elements which have <b>one</b> mechanical frame,
which is called frame_a.
</p>
</HTML>
"));
      public 
        Frame_a frame_a(
          S=Sa,
          r0=r0a,
          v=va,
          w=wa,
          a=aa,
          z=za,
          f=fa,
          t=ta) annotation (extent=[-120, -16; -90, 14]);
      end OneFrame_a;
      
      partial model OneFrame_b 
        "Superclass of elements with ONE mechanical frame_b" 
        
      protected 
        Real Sb[2, 2](start=identity(2));
        SIunits.Position r0b[2];
        SIunits.Velocity vb[2];
        SIunits.AngularVelocity wb;
        SIunits.Acceleration ab[2];
        SIunits.AngularAcceleration zb;
        SIunits.Force fb[2];
        SIunits.Torque tb;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42),
          Documentation(info="
Superclass of elements which have ONE mechanical frame,
which is called frame_b .
"),       Diagram);
        
      public 
        Frame_b frame_b(
          S=Sb,
          r0=r0b,
          v=vb,
          w=wb,
          a=ab,
          z=zb,
          f=-fb,
          t=-tb) annotation (extent=[90, -16; 120, 16]);
      end OneFrame_b;
      
      model TwoTreeFrames 
        "Superclass of elements of the spanning tree with TWO frames" 
        
      protected 
        Real Sa[2, 2](start=identity(2));
        SIunits.Position r0a[2];
        SIunits.Velocity va[2];
        SIunits.AngularVelocity wa;
        SIunits.Acceleration aa[2];
        SIunits.AngularAcceleration za;
        SIunits.Force fa[2];
        SIunits.Torque ta;
        
        Real Sb[2, 2](start=identity(2));
        SIunits.Position r0b[2];
        SIunits.Velocity vb[2];
        SIunits.AngularVelocity wb;
        SIunits.Acceleration ab[2];
        SIunits.AngularAcceleration zb;
        SIunits.Force fb[2];
        SIunits.Torque tb;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42),
          Documentation(info="<HTML>
<p>
Superclass of elements which have <b>two</b> mechanical frames in the
spanning tree, which are called <b>frame_a</b> and 
<b>frame_b</b>, respectively.
<p>
<b>Important</b><br>
frame_a of an element should <b>always</b> be connected to a frame_b.<br>
frame_b of an element should <b>always</b> be connected to a frame_a.
</p>
</HTML>
"),       Diagram);
        
      public 
        Frame_b frame_b(
          S=Sb,
          r0=r0b,
          v=vb,
          w=wb,
          a=ab,
          z=zb,
          f=-fb,
          t=-tb) annotation (extent=[90, -16; 120, 16]);
        Frame_a frame_a(
          S=Sa,
          r0=r0a,
          v=va,
          w=wa,
          a=aa,
          z=za,
          f=fa,
          t=ta) annotation (extent=[-120, -16; -90, 16]);
      end TwoTreeFrames;
      
      model BodyBase "Inertia and mass properties of a rigid body" 
        extends OneFrame_a;
        SIunits.Mass m;
        SIunits.Position rCM[2];
        SIunits.Inertia I;
        
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(Rectangle(extent=[-40, 60; 80, -60], style(gradient=3, fillColor=69)),
               Rectangle(extent=[-90, 14; -40, -16], style(gradient=2, fillColor=69))),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42),
          Diagram(
            Rectangle(extent=[-90, 15; 0, -15], style(
                color=8,
                gradient=2,
                fillColor=75,
                fillPattern=1)),
            Rectangle(extent=[0, 50; 100, -50], style(
                color=8,
                gradient=3,
                fillColor=75,
                fillPattern=1)),
            Text(extent=[22, -64; 82, -104], string="center of mass"),
            Line(points=[-90, -10; -90, -72]),
            Line(points=[50, 0; 50, -72]),
            Line(points=[-90, -66; 50, -66]),
            Polygon(points=[36, -62; 36, -70; 50, -66; 36, -62], style(fillColor=3,
                   fillPattern=1)),
            Text(extent=[-50, -44; -24, -62], string="rCM")),
          Documentation(info="<HTML>
<p>
This model should usually not be used directly, because the mass
properties have to be given as terminal variables and not as
parameters. This allows the computation of the mass properties
from other data, as well as the modification of the mass properties
at event points. The following variables have to be computed in
subclasses:
</p>
<pre>
  m     : Mass of body in [kg].
  rCM(2): Position vector from the origin of frame_a to the center
          of mass, resolved in frame_a in [m].
  I     : Inertia tensor of the body with respect to the center of mass,
          resolved in frame_a in [kgm^2].
</pre>
</HTML>
"));
        
      equation 
        fa = m*(aa + {-za*rCM[2],za*rCM[1]} + {-wa*wa*rCM[1],wa*(-wa)*rCM[2]});
        // m*(aa + cross(za, rCM) + cross(wa, cross(wa, rCM)));
        ta = I*za + 0 + rCM[1]*fa[2] - rCM[2]*fa[1];
        //I*za+ cross(wa, I*wa) + cross(rCM, fa);
      end BodyBase;
      
      partial model TreeJoint "Superclass of joints used in the spanning tree" 
        extends TwoTreeFrames;
        Real S_rel[2, 2];
        SIunits.Position r_rela[2];
        SIunits.Velocity v_rela[2];
        SIunits.AngularVelocity w_rela;
        SIunits.Acceleration a_rela[2];
        SIunits.AngularAcceleration z_rela;
        annotation (
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Window(
            x=0.45,
            y=0.01,
            width=0.53,
            height=0.5),
          Documentation(info="<HTML>
<p>
A joint has two mechanical cuts which may move relative to each other. In
every cut a coordinate system is rigidly attached, called frame_a and frame_b,
correspondingly. It is a massless element in which no energy is stored. 
Mathematically, a joint transforms the kinematic properties of frame_a to 
frame_b and at the same time transforms the force and torque acting on 
frame_b to frame_a.
</p>
<p>
A general multibody system with closed kinematic loops is handeled by dividing
the joints into two distinct sets: <b>Tree-Joints</b> and <b>Cut-Joints</b>. 
After removal of all of the Cut-Joints, the resulting system must have a 
tree-structure. Class TreeJoint is used as a superclass for Tree-Joints.
</p>
<p>
The relative motion between the two cut-frames of a Tree-Joint is described
by f (0 <= f <= 3) generalized minimal-coordinates q and their first and
second derivatives qd, qdd. In subclasses of class TreeJoint the relative
kinematic quantities are given as functions of q, qd, qdd, according to the
specific joint type. In class TreeJoint the relationships are provided
between the kinematic and dynamic quantities of frame_a and frame_b and of the
relative quantities.
</p>
<p>
In order to speedup the generation of the equations, the common equations of 
TreeJoint classes are <b>not</b> stored in model TreeJoint, but in the 
specific submodel. This has the advantage that special joint properties 
(like S_rel=identity(2), i.e., the relative transformation matrix is a 
unit matrix) are already utilized and the Modelica translator does not have 
to waste time and space to find this out by symbolic formula transformation. 
The common equations which could be stored in the TreeJoint 
model are given as a comment below.
</p>
"));
      equation 
        /* Equations to relate quantities of frame_a and frame_b.
     These equations are defined in the submodels and are
     adapted to the particular joint.
    Sb = Sa*transpose(S_rel);
    r0b = r0a + Sa*r_rela;
    vaux = cross(wa,r_rela)
    vb   = S_rel*(va + v_rela + vaux)
    wb   = S_rel*(wa + w_rela)
    ab   = S_rel*( aa + a_rela + cross(za,r_rela) + cross(wa,vaux+2*v_rela) )
    zb   = S_rel*( za + z_rela + cross(wa,w_rela) )
    ab = S_rel*aa;
    zb = S_rel*(za + z_rela + cross(wa, w_rela));
    fa = transpose(S_rel)*fb;
    ta = transpose(S_rel)*tb + cross(r_rela,fa);
*/
      end TreeJoint;
    end Interfaces;
    
    package Joints 
      extends Modelica.Icons.Library;
      annotation (
        Coordsys(
          extent=[0, 0; 442, 386],
          grid=[1, 1],
          component=[20, 20]),
        Window(
          x=0,
          y=0.6,
          width=0.4,
          height=0.4,
          library=1,
          autolayout=1),
        Documentation(info="<HTML>
<p>This package contains joint models.</p>
</HTML>"));
      
      model Revolute 
        "Revolute joint (1 degree-of-freedom, used in spanning tree)" 
        
        extends Interfaces.TreeJoint;
        parameter Real q0=0 "Rotation angle offset (see info) [deg]";
        parameter Boolean startValueFixed=false 
          "true, if start values of q, qd are fixed";
        SIunits.Angle q(final fixed=startValueFixed);
        SIunits.AngularVelocity qd(final fixed=startValueFixed);
        SIunits.AngularAcceleration qdd;
        SIunits.Angle qq;
        Real sinq;
        Real cosq;
        constant Real PI=Modelica.Constants.pi;
        
        annotation (
          preferedView="info",
          defaultComponentName="rev",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Rectangle(extent=[-90, -60; -30, 60], style(
                color=8,
                gradient=2,
                fillColor=8)),
            Rectangle(extent=[30, -60; 90, 60], style(
                color=8,
                gradient=2,
                fillColor=8,
                fillPattern=1)),
            Rectangle(extent=[-30, 10; 10, -10], style(
                color=0,
                gradient=0,
                fillColor=8,
                fillPattern=1)),
            Polygon(points=[10, 30; 30, 50; 30, -50; 10, -30; 10, 30], style(
                color=0,
                fillColor=8,
                fillPattern=1)),
            Line(points=[-20, 70; -50, 70], style(color=0)),
            Line(points=[-20, 80; -20, 60], style(color=0)),
            Line(points=[20, 80; 20, 60], style(color=0)),
            Line(points=[20, 70; 41, 70], style(color=0)),
            Rectangle(extent=[-10, 60; 10, 50], style(
                color=8,
                gradient=1,
                fillColor=8)),
            Polygon(points=[-10, 30; 10, 30; 30, 50; -30, 50; -10, 30], style(
                color=0,
                fillColor=8,
                fillPattern=1))),
          Icon(
            Rectangle(extent=[-90, -60; -30, 60], style(
                color=8,
                gradient=2,
                fillColor=8)),
            Rectangle(extent=[30, -60; 90, 60], style(
                color=8,
                gradient=2,
                fillColor=8,
                fillPattern=1)),
            Rectangle(extent=[-30, 10; 10, -10], style(
                color=0,
                gradient=0,
                fillColor=8,
                fillPattern=1)),
            Polygon(points=[10, 30; 30, 50; 30, -50; 10, -30; 10, 30], style(
                color=0,
                fillColor=8,
                fillPattern=1)),
            Line(points=[-20, 70; -50, 70], style(color=0)),
            Line(points=[-20, 80; -20, 60], style(color=0)),
            Line(points=[20, 80; 20, 60], style(color=0)),
            Line(points=[20, 70; 41, 70], style(color=0)),
            Rectangle(extent=[-10, 60; 10, 50], style(
                color=8,
                gradient=1,
                fillColor=8)),
            Polygon(points=[-10, 30; 10, 30; 30, 50; -30, 50; -10, 30], style(
                color=0,
                fillColor=8,
                fillPattern=1)),
            Text(extent=[-100, -138; 100, -79], string="%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.45,
            height=0.46),
          Documentation(info="<HTML>
<p>
Joint where frame_b rotates around axis perpedicular to model plane which is fixed in frame_a.
The joint axis has an additional flange where it can be
driven with elements of the Modelica.Mechanics.Rotational library.
The relative angle q [rad] and the relative angular velocity 
qd [rad/s] are used as state variables.
</p>
<p>
The following parameters are used to define the joint:
</p>
<pre>
  q0: Rotation angle offset in [deg].
      If q=q0, frame_a and frame_b are identical.
  startValueFixed: true, if start values of q, qd are fixed.
</pre>
</HTML>
"));
        Modelica.Mechanics.Rotational.Interfaces.Flange_a axis 
          annotation (extent=[10, 80; -10, 60]);
        Modelica.Mechanics.Rotational.Interfaces.Flange_b bearing 
          annotation (extent=[-70, 80; -50, 60]);
      equation 
        axis.phi = q;
        bearing.phi = 0;
        
        // define states
        qd = der(q);
        qdd = der(qd);
        
        /*rotation matrix*/
        qq = q - q0*PI/180;
        sinq = Modelica.Math.sin(qq);
        cosq = Modelica.Math.cos(qq);
        S_rel = [cosq, sinq; -sinq, cosq];
        
          //[nn]*transpose([nn])+(identity(3)-[nn]*transpose([nn]))*cosq-skew(nn)*sinq;
        
        /*other kinematic quantities*/
        r_rela = zeros(2);
        v_rela = zeros(2);
        a_rela = zeros(2);
        w_rela = qd;
        z_rela = qdd;
        
        /* Transform the kinematic quantities from frame_a to frame_b and the
     force and torque acting at frame_b to frame_a
     (= general equations of a "TreeJoint" specialized to this class).
  */
        Sb = Sa*transpose(S_rel);
        r0b = r0a;
        
        vb = S_rel*va;
        wb = (wa + w_rela);
        
        ab = S_rel*aa;
        zb = (za + z_rela);
        
        fa = transpose(S_rel)*fb;
        ta = tb;
        
        // d'Alemberts principle
        axis.tau = tb;
      end Revolute;
      
      model FreeMotion 
        "Free motion joint (3 degrees-of-freedom, used in spanning tree)" 
        
        extends Interfaces.TreeJoint;
        
          // S_rel needs a correct start value, because pre(S_rel) is referenced below
        SIunits.Angle phi 
          "Cardan angles from a frame fixed in frame_a to frame_b";
        
      protected 
        constant Real phi2_critical_deg=80 
          "angle in [deg] too close to singularity. Redefine S_fix and phi";
        constant Real phi2_critical=phi2_critical_deg*Modelica.Constants.pi/180.0;
        SIunits.Velocity vaux[2];
        Real s3;
        Real c3;
        //Boolean switch_state;
        //Real S_phi[2, 2] "S_rel = S_phi(phi)*S_fix";
        //Real S_fix[2, 2](start=identity(2)) "S_rel = S_phi(phi)*S_fix";
        
        annotation (
          preferedView="info",
          defaultComponentName="free",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8)),
            Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8)),
            Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)),
            Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)),
            Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0,
                  fillColor=8)),
            Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0,
                  fillColor=8)),
            Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 48],
                 style(
                color=9,
                thickness=2,
                fillColor=8))),
          Icon(
            Rectangle(extent=[-70, -5; -90, 5], style(color=0, fillColor=8)),
            Rectangle(extent=[-30, -5; -50, 5], style(color=0, fillColor=8)),
            Rectangle(extent=[11, -5; -9, 5], style(color=0, fillColor=8)),
            Rectangle(extent=[50, -5; 30, 5], style(color=0, fillColor=8)),
            Polygon(points=[90, 0; 50, 20; 50, -20; 90, 0], style(color=0,
                  fillColor=8)),
            Polygon(points=[69, 58; 49, 40; 77, 28; 69, 58], style(color=0,
                  fillColor=8)),
            Line(points=[-86, 31; -74, 61; -49, 83; -17, 92; 19, 88; 40, 69; 59, 48],
                 style(
                color=9,
                thickness=2,
                fillColor=8)),
            Text(
              extent=[100, -40; -100, -100],
              string="%name",
              style(color=3))),
          Window(
            x=0.25,
            y=0.38,
            width=0.51,
            height=0.68),
          Documentation(info="<HTML> 
<p>
Joint which does not constrain the motion between frame_a and frame_b.
Such a joint is just used to define the desired states to be used.
The following state variables are used:
</p>
<pre>
  r_rela[2]: Distance vector from the origin of frame_a to the origin
             of frame_b, resolved in frame_a in [m].
  phi      : Rotation angle between frame_a and frame_b, perpendicular 
             to the plane [rad].
  v_rela[2]: = der(r_rela); relative velocity of frame_b with respect to
             frame_a resolved in frame_a in [m/s].
  w_rela   : Relative angular velocity of frame_b with respect to frame_a
             resolved in frame_a in [rad/s].
</pre>
</HTML>
"));
      equation 
        
        /* Determine sines and cosines of the Cardan angles */
        s3 = Modelica.Math.sin(phi);
        c3 = Modelica.Math.cos(phi);
        
        /*switch_state = phi >= phi2_critical or phi <= -phi2_critical;
  when switch_state then
    S_fix = pre(S_rel);
    reinit(phi, 0);
  end when;
  S_phi = [c3, s3; -s3, c3];
  S_rel = S_phi*S_fix;
  */
        S_rel = [c3, s3; -s3, c3];
        
        // Kinematic differential equations for translational motion
        der(r_rela) = v_rela;
        der(v_rela) = a_rela;
        
        // Kinematic differential equations for rotational motion
        der(phi) = w_rela;
        der(w_rela) = z_rela;
        
        // Kinematic relationships
        frame_b.S = frame_a.S*transpose(S_rel);
        frame_b.r0 = frame_a.r0 + frame_a.S*r_rela;
        
        vaux = {-frame_a.w*r_rela[2],frame_a.w*r_rela[1]};
        //cross(frame_a.w, r_rela);
        frame_b.v = S_rel*(frame_a.v + v_rela + vaux);
        frame_b.w = (frame_a.w + w_rela);
        
        frame_b.a = S_rel*(frame_a.a + {-frame_a.z*r_rela[2],frame_a.z*r_rela[1]}
           + {-frame_a.w*(vaux[2] + 2*v_rela[2]),frame_a.w*(vaux[1] + 2*v_rela[1])}
           + a_rela);
        
          //S_rel*(frame_a.a + cross(frame_a.z, r_rela) + cross(frame_a.w,vaux + 2*v_rela) + a_rela);
        zb = (frame_a.z + z_rela);
        
        //frame_b.z = S_rel*(frame_a.z + cross(frame_a.w, w_rela) + z_rela);
        
        // cut-forces and cut-torques are zero
        fa = zeros(2);
        ta = 0;
        fb = zeros(2);
        tb = 0;
      end FreeMotion;
    end Joints;
    
    package Parts 
      extends Modelica.Icons.Library;
      annotation (
        Coordsys(
          extent=[0, 0; 442, 394],
          grid=[1, 1],
          component=[20, 20]),
        Window(
          x=0.45,
          y=0.01,
          width=0.35,
          height=0.49,
          library=1,
          autolayout=1),
        Documentation(info="<HTML>
<p>This package contains models for rigid parts.</p>
</HTML>"));
      
      model InertialSystem 
        extends Interfaces.OneFrame_b;
        parameter SIunits.Acceleration g=9.81 "Gravity constant";
        parameter Real ng[2]={0,0} "Direction of gravity (gravity = g*ng)";
        parameter String label1="x" "Label of horizontal axis in icon";
        parameter String label2="y" "Label of vertical axis in icon";
        SIunits.Acceleration gravity[2] "Gravity acceleration vector";
        annotation (
          preferedView="info",
          defaultComponentName="ground",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Diagram(
            Line(points=[-80, -100; -80, 30], style(
                color=0,
                pattern=5,
                thickness=2,
                fillColor=6,
                fillPattern=1)),
            Polygon(points=[-80, 70; -100, 30; -60, 30; -80, 70; -80, 70], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Line(points=[-100, -80; 50, -80], style(
                color=0,
                thickness=2,
                fillColor=0,
                fillPattern=1)),
            Polygon(points=[90, -80; 50, -60; 50, -100; 90, -80], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Line(points=[-80, 70; 90, 70; 90, -80], style(color=0)),
            Text(extent=[0, 146; 0, 75], string="%name"),
            Line(points=[-80, -100; -80, 30], style(
                color=0,
                pattern=5,
                thickness=2,
                fillColor=6,
                fillPattern=1)),
            Polygon(points=[-80, 70; -100, 30; -60, 30; -80, 70; -80, 70], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Line(points=[-100, -80; 50, -80], style(
                color=0,
                thickness=2,
                fillColor=0,
                fillPattern=1)),
            Polygon(points=[90, -80; 50, -60; 50, -100; 90, -80], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Line(points=[-80, 70; 90, 70; 90, -80], style(color=0)),
            Text(extent=[50, 17; 50, -52], string="%label1"),
            Text(extent=[-30, 60; -30, -11], string="%label2")),
          Icon(
            Line(points=[-80, -100; -80, 30], style(
                color=0,
                pattern=5,
                thickness=2,
                fillColor=6,
                fillPattern=1)),
            Polygon(points=[-80, 70; -100, 30; -60, 30; -80, 70; -80, 70], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Line(points=[-100, -80; 50, -80], style(
                color=0,
                thickness=2,
                fillColor=0,
                fillPattern=1)),
            Polygon(points=[90, -80; 50, -60; 50, -100; 90, -80], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Line(points=[-80, 70; 90, 70; 90, -80], style(color=0)),
            Text(extent=[0, 146; 0, 75], string="%name"),
            Line(points=[-80, -100; -80, 30], style(
                color=0,
                pattern=5,
                thickness=2,
                fillColor=6,
                fillPattern=1)),
            Polygon(points=[-80, 70; -100, 30; -60, 30; -80, 70; -80, 70], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Line(points=[-100, -80; 50, -80], style(
                color=0,
                thickness=2,
                fillColor=0,
                fillPattern=1)),
            Polygon(points=[90, -80; 50, -60; 50, -100; 90, -80], style(
                color=0,
                fillColor=0,
                fillPattern=1)),
            Line(points=[-80, 70; 90, 70; 90, -80], style(color=0)),
            Text(extent=[50, 17; 50, -52], string="%label1"),
            Text(extent=[-30, 60; -30, -11], string="%label2")),
          Window(
            x=0.24,
            y=0.22,
            width=0.6,
            height=0.6));
      equation 
        gravity = g*ng;
        
        /*Equations*/
        frame_b.S = identity(2);
        frame_b.r0 = zeros(2);
        frame_b.v = zeros(2);
        frame_b.w = 0;
        frame_b.a = -gravity;
        frame_b.z = 0;
      end InertialSystem;
      
      model FrameTranslation 
        "Fixed translation of frame_b with respect to frame_a" 
        
        //+cross(za, r) + cross(wa, vaux);
        
        /*Transform the force and torque acting at frame_b to frame_a*/
        //+ cross(r, fa);
        extends Interfaces.TwoTreeFrames;
        parameter SIunits.Position r[2]={0,0} 
          "Vector from frame_a to frame_b resolved in frame_a";
      protected 
        SIunits.Velocity vaux[2];
        annotation (
          preferedView="info",
          defaultComponentName="transl",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[1, 1],
            component=[20, 20]),
          Window(
            x=0.25,
            y=0.04,
            width=0.75,
            height=0.65),
          Documentation(info="
Fixed translation of frame_b with respect to frame_a, i.e., the
frames of connectors a and b are parallel to each other.
Parameters:
  r: Position vector from the origin of (connector) frame_a to the origin of
     (connector) frame_b, resolved in frame_a in [m].
"),       Icon(
            Text(
              extent=[-99, -100; 101, -40],
              string="r=%r",
              style(color=0)),
            Rectangle(extent=[-90, 5; 90, -5], style(color=0, fillColor=0)),
            Text(extent=[-101, 41; 99, 101], string="%name")),
          Diagram(
            Line(points=[-90, 0; -90, -65]),
            Line(points=[90, 0; 90, -65]),
            Line(points=[-90, -60; 80, -60]),
            Polygon(points=[80, -55; 80, -65; 90, -60; 80, -55], style(fillColor=3,
                   fillPattern=1)),
            Text(extent=[-22, -36; 16, -60], string="r"),
            Rectangle(extent=[-90, 5; 90, -5], style(color=0, fillColor=0)),
            Line(points=[69, 29; 106, 29], style(arrow=1)),
            Line(points=[70, 27; 70, 59], style(arrow=1)),
            Text(extent=[92, 44; 106, 33], string="x"),
            Text(extent=[51, 66; 66, 54], string="y"),
            Line(points=[-103, 29; -66, 29], style(arrow=1)),
            Line(points=[-102, 27; -102, 59], style(arrow=1)),
            Text(extent=[-80, 44; -66, 33], string="x"),
            Text(extent=[-121, 66; -106, 54], string="y")));
      equation 
        
        Sb = Sa;
        wb = wa;
        zb = za;
        
        r0b = r0a + Sa*r;
        vaux = {-wa*r[2],wa*r[1]};
        //vaux = cross(wa, r);
        vb = va + vaux;
        ab = aa + {-za*r[2],za*r[1]} + {-wa*vaux[2],wa*vaux[1]};
        //+cross(za, r) + cross(wa, vaux);
        
        /*Transform the force and torque acting at frame_b to frame_a*/
        fa = fb;
        ta = tb + r[1]*fa[2] - r[2]*fa[1];
        //+ cross(r, fa);
      end FrameTranslation;
      
      model Body "Rigid body with one cut" 
        extends Interfaces.OneFrame_a;
        parameter SIunits.Position rCM[2]={0,0} 
          "Vector from frame_a to center of mass, resolved in frame_a";
        parameter SIunits.Mass m=0 "Mass of body [kg]";
        parameter SIunits.Inertia I=0 "inertia";
        annotation (
          preferedView="info",
          defaultComponentName="body",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[-90, 14; -40, -16], style(gradient=2, fillColor=69)),
            Rectangle(extent=[-40, 60; 80, -60], style(gradient=3, fillColor=69)),
            Text(extent=[-98, 124; 102, 60], string="%name")),
          Diagram(Line(points=[-86, 22; -24, 22], style(color=0, arrow=1)), Text(
              extent=[-80, 48; -32, 26],
              string="rCM",
              style(color=0))),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42),
          Documentation(info="
Rigid body with one cut. All the mass and inertia properties must be
supplied with respect to the center of mass.
Parameters:
  rCM[2]: Position vector from the origin of frame_a to the center of 
          mass, resolved in frame_a in [m].
  m     : Mass of body in [kg].
  I     : Inertia.
"));
        
        Interfaces.BodyBase body annotation (extent=[-20, -40; 60, 40]);
      equation 
        connect(frame_a, body.frame_a) annotation (points=[-105, -1; -84.75, -1; -84.75,
               -1.4; -64.5, -1.4; -64.5, -0.4; -22, -0.4], style(color=0, thickness=
               4));
        body.m = m;
        body.rCM = rCM;
        body.I = I;
      end Body;
      
      model Body2 "Rigid body with two cuts" 
        extends Interfaces.TwoTreeFrames;
        parameter SIunits.Position r[2]={0,0} 
          "Vector from frame_a to frame_b, resolved in frame_a";
        parameter SIunits.Position rCM[2]={0,0} 
          "Vector from frame_a to center of mass, resolved in frame_a";
        parameter SIunits.Mass m=0 "Mass of body";
        parameter SIunits.Inertia I=0 "inertia";
        annotation (
          preferedView="info",
          defaultComponentName="body",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[-60, 60; 60, -60], style(gradient=3, fillColor=69)),
            Rectangle(extent=[-90, 14; -60, -16], style(gradient=2, fillColor=69)),
            Rectangle(extent=[60, 14; 90, -16], style(gradient=2, fillColor=69)),
            Text(extent=[-100, 118; 102, 60], string="%name")),
          Window(
            x=0.45,
            y=0.01,
            width=0.28,
            height=0.42),
          Documentation(info="
Rigid body with two frames. All the mass and inertia properties must be
supplied with respect to the center of mass. All parameter vectors
have to be resolved in frame_a in [m].
Parameters:
  r[3]  : Position vector from the origin of frame_a to the origin of 
          frame_b.
  rCM[3]: Position vector from the origin of frame_a to the center of 
          mass.
  m     : Mass of body in [kg].
  Ixy   : Elements (x,y) of the inertia tensor of the body with respect to
          the center of mass, resolved frame_a in [kgm^2]. 
          The following elements can be defined:
          I11, I22, I33, I21, I31, I32.
          (Move the inertial from into the center of mass by translation
          and resolve the inertia tensor in this frame).
"));
        
        FrameTranslation frameTranslation(r=r) annotation (extent=[-30, 20; 30, 80]);
        Body body(
          rCM=rCM,
          m=m,
          I=I) annotation (extent=[-30, -80; 30, -20]);
      equation 
        connect(frameTranslation.frame_b, frame_b) 
          annotation (points=[31.5, 50; 71, 50; 71, 0; 105, 0], style(color=0));
        connect(body.frame_a, frame_a) annotation (points=[-31.5, -50.3; -60, -50.3;
               -60, 0; -105, 0], style(color=0));
        connect(frameTranslation.frame_a, frame_a) annotation (points=[-31.5, 50; -60,
               50; -60, 0; -105, 0], style(color=0));
        
      end Body2;
      
      model TwoD2ThreeD 
        parameter SIunits.Position[3] x0={1,0,0} 
          "Direction of 2D x-axis resolved in 3D inertial system";
        parameter SIunits.Position[3] y0={0,1,0} 
          "Direction of 2D y-axis resolved in 3D inertial system";
        parameter SIunits.Velocity vz0=0 
          "Speed perpendicular to the x-y plane of 2d cut";
        parameter SIunits.Position[3] r0={0,0,0} 
          "Origin of the 2D-plane, resolved in InertialFrame3D";
      protected 
        parameter Real[3] ex=x0/sqrt(x0*x0);
        parameter Real[3] ey=y0/sqrt(y0*y0);
        parameter Real[3, 3] S_rel=transpose([ex, ey, cross(ex, ey)]);
        
        Real Sb[3, 3](start=identity(3));
        SIunits.Position r0b[3];
        SIunits.Velocity vb[3];
        SIunits.AngularVelocity wb[3];
        SIunits.Acceleration ab[3];
        SIunits.AngularAcceleration zb[3];
        SIunits.Force fb[3];
        SIunits.Torque tb[3];
        
        Real Sa[2, 2](start=identity(2));
        SIunits.Position r0a[2];
        SIunits.Velocity va[2];
        SIunits.AngularVelocity wa;
        SIunits.Acceleration aa[2];
        SIunits.AngularAcceleration za;
        SIunits.Force fa[2];
        SIunits.Torque ta;
        
        SIunits.Force faux[3] "auxilliary force";
        SIunits.Torque taux[3] "auxilliary torque";
        annotation (
          preferedView="info",
          defaultComponentName="conv",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Text(extent=[-100, 100; 100, 36], string="%name"),
            Polygon(points=[66, 6; 66, -4; 76, 2; 66, 6], style(color=0, fillColor=
                    0)),
            Rectangle(extent=[-80, 2; -56, 0], style(color=0, fillColor=0)),
            Rectangle(extent=[-44, 2; -20, 0], style(color=0, fillColor=0)),
            Rectangle(extent=[-6, 2; 18, 0], style(color=0, fillColor=0)),
            Rectangle(extent=[32, 2; 56, 0], style(color=0, fillColor=0))),
          Window(
            x=0.05,
            y=0.19,
            width=0.75,
            height=0.76),
          Diagram(Text(extent=[-146, -46; 44, -128], string="Contains bugs")));
        
      public 
        Modelica.Mechanics.MultiBody.Interfaces.Frame_b frame_b(
          transpose(R)=
            Sb,
          r_0=
             r0b,
          Modelica(Mechanics(MultiBody(Frames(
          resolve2( frame_bYY(                         R),          der(
                        frame_bYY(                                      r_0)))=
            vb,
          angularVelocity2(
                    frame_bYY(                                 R),          der(
                        frame_bYY(                                              R)))=
            wb)))),
          a=ab,
          z=zb,
          f=-fb,
          t=-tb) annotation (extent=[90, -15; 120, 15]);
        Interfaces.Frame_a frame_a(
          S=Sa,
          r0=r0a,
          v=va,
          w=wa,
          a=aa,
          z=za,
          f=fa,
          t=ta) annotation (extent=[-120, -16; -90, 14]);
      equation 
        /*Adding an extra dimension to Sa in the z direction:
To transform frame_a into frame_b via InertialFrame3D
        Sa3=[extended as below]
    Sb=S*Sa3*transpose(S_rel);
    Sa2b=transpose(Sb)*S*Sa3=S*transpose(Sa3)*transpose(S)*S*Sa3=
    S*transpose(Sa3)*Sa3=S
*/
        Sb = transpose(S_rel)*[Sa[1, 1], Sa[1, 2], 0; Sa[2, 1], Sa[2, 2], 0; 0, 0,
          1]*S_rel;
        r0b = r0 + transpose(S_rel)*{r0a[1],r0a[2],vz0*time}*S_rel;
        vb = transpose(S_rel)*{va[1],va[2],vz0}*S_rel;
        ab = transpose(S_rel)*{aa[1],aa[2],0}*S_rel;
        wb = transpose(S_rel)*{0,0,wa}*S_rel;
        zb = transpose(S_rel)*{0,0,za}*S_rel;
        faux = (S_rel*fb*S_rel);
        taux = (S_rel*tb*S_rel);
        fa = {faux[1],faux[2]};
        ta = taux[3];
      end TwoD2ThreeD;
      
      model ShapeBody "Rigid body with visual shape (also used for animation)" 
        import VEHICLE;
        import BasicVehicleSystems.Chassis.BaseClasses.PlanarMultiBody;
        extends Interfaces.TwoTreeFrames;
        
        parameter SIunits.Position r[2]={0.1,0} 
          "Vector from frame_a to frame_b, resolved in frame_a";
        parameter SIunits.Position rCM[2]={0,0} 
          "Vector from frame_a to center of mass, resolved in frame_a";
        parameter SIunits.Mass m=0 "Mass of body";
        parameter SIunits.Inertia I=0 "Inertia";
        
        parameter SIunits.Position[3] x0={1,0,0} 
          "Direction of 2D x-axis resolved in 3D inertial system";
        parameter SIunits.Position[3] y0={0,1,0} 
          "Direction of 2D y-axis resolved in 3D inertial system";
        parameter SIunits.Position[3] r0={0,0,0} 
          "Origin of the 2D-plane, resolved in InertialFrame3D";
        
        parameter String Shape="box" "Name of shape (see info text)";
        parameter SIunits.Position ra[3]={0,0,0} 
          "Vector from frame_a to shape origin. resolved in frame_a";
        parameter SIunits.Position LengthDirection[3]=r[1]*x0 + r[2]*y0 - ra 
          "Vector in length direction, resolved in frame_a";
        parameter SIunits.Position WidthDirection[3]={0,1,0} 
          "Vector in width direction, resolved in frame_a";
        parameter SIunits.Length Length=(sqrt((r[1]*x0 + r[2]*y0 - ra)*(r[1]*x0 + r[
            2]*y0 - ra))) "Length of shape";
        parameter SIunits.Length Width=0.1 "Width of shape";
        parameter SIunits.Length Height=0.1 "Height of shape";
        parameter Real Material[4]={1,0,0,0.5} "Color and specular coefficient";
        parameter Real Extra=0.0 "Additional parameter for cone and pipe";
        PlanarMultiBody.Parts.Shape vis(
          x0=x0,
          y0=y0,
          r0=r0,
          Shape=Shape,
          ra=ra,
          LengthDirection=LengthDirection,
          WidthDirection=WidthDirection,
          Length=Length,
          Width=Width,
          Height=Height,
          Material=Material,
          Extra=Extra) annotation (extent=[-20, -100; 40, -40]);
        annotation (
          preferedView="info",
          defaultComponentName="body",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Rectangle(extent=[-90, 14; -48, -16], style(
                color=8,
                gradient=2,
                fillColor=75,
                fillPattern=1)),
            Polygon(points=[-50, 34; -50, -60; 0, -40; 40, -60; 40, 36; 0, 12; -50,
                   34], style(
                color=7,
                fillColor=46,
                fillPattern=1)),
            Polygon(points=[-50, 34; -30, 46; 10, 30; 60, 62; 40, 36; 0, 12; -50,
                  34], style(
                color=7,
                fillColor=75,
                fillPattern=1)),
            Polygon(points=[58, 60; 40, 36; 40, -60; 58, -36; 58, 60], style(
                color=7,
                fillColor=75,
                fillPattern=1)),
            Rectangle(extent=[48, 14; 90, -16], style(
                color=8,
                gradient=2,
                fillColor=75,
                fillPattern=1)),
            Text(extent=[-100, 120; 100, 61], string="%name"),
            Text(
              extent=[-100, -61; 100, -99],
              string="%Shape",
              style(color=0))),
          Window(
            x=0.55,
            y=0.21,
            width=0.28,
            height=0.42),
          Documentation(info="
Rigid body with two cuts and an additional shape used in animation.
All the mass and inertia properties must be supplied with respect
to the center of mass. All parameter vectors are resolved in frame_a
in [m].
Parameters:
  r[2]         : Position vector from the origin of frame_a to the origin of 
                 frame_b.
  rCM[2]       : Position vector from the origin of frame_a to the center of 
                 mass.
  m            : Mass of body in [kg].
  I            : Inerita
  Shape        : Type of visual shape. Predefined shapes are
                   Shape = \"box\" 
                           \"sphere\" 
                           \"cylinder\" 
                           \"cone\" 
                           \"pipe\" 
                           \"beam\"
                           \"gearwheel\"
                           \"wirebox\" 
                 External shapes are specified as DXF-files (only 3DFace is supported).
                 External shapes must be named \"1\", \"2\"  etc.. The corresponding definitions 
                 should be in files \"1.dxf\", \"2.dxf\" etc.Since the DXF-files contain color
                 and dimensions for the individual faces, the corresponding information
                 in the model is currently ignored, but the specular coefficient is 
                 utilized. The DXF-files must be found in the current directory. 
  r0[3]        : Position vector from the origin of frame_a to the origin
                 of the visual object.
  LengthDirection[3]: Unit vector in direction of length (will be normalized).
  WidthDirection[3] : Unit vector in direction of width (will be normalized).
  Length       : Length of visual shape in [m].
  Width        : Width of visual shape in [m].
  Height       : Height of visual shape in direction of a vector which is
                 orthogonal to  LengthDirection  and  WidthDirection.
  Material[4]  : = {r, g, b, specular} specifies the color of the shape.
                 [r,g,b] affects the color of diffuse and ambient reflected
                 light. Specular is a coefficient defining white specular
                 reflection. What color that is reflected also depends on the
                 color of the light sources. Note, r g, b and specular are
                 given in the range 0 .. 1. Specular=1 gives a metallic appearance.
  Extra        : Additional parameter for cone and pipe.
                 \"pipe\"     : A hollow cylinder. Extra = InnerRadius/Radius of the cylinder.
                 \"gearwheel\": A ring with gear wheel teeth. Extra = number of teeth.
"));
        Body2 body(
          r=r,
          rCM=rCM,
          m=m,
          I=I) annotation (extent=[-40, -40; 40, 40]);
      equation 
        connect(body.frame_a, frame_a) annotation (points=[-42, 0; -59, 0; -59, -1.4;
               -76, -1.4; -76, 0; -105, 0], style(color=10));
        connect(body.frame_b, frame_b) annotation (points=[42, 0; 57.5, 0; 57.5, -1.4;
               73, -1.4; 73, 0; 105, 0], style(color=0));
        connect(vis.frame_a, frame_a) annotation (points=[-21.5, -70.3; -61, -70.3;
               -61, 0; -105, 0], style(color=10));
      end ShapeBody;
      
      model Shape "General animation shape" 
        extends Interfaces.OneFrame_a;
        
        parameter SIunits.Position[3] x0={1,0,0} 
          "Direction of 2D x-axis resolved in 3D inertial system";
        parameter SIunits.Position[3] y0={0,1,0} 
          "Direction of 2D y-axis resolved in 3D inertial system";
        parameter SIunits.Position[3] r0={0,0,0} 
          "Origin of the 2D-plane, resolved in InertialFrame3D";
        
        parameter String Shape="box" "Name of shape (see info text)";
        parameter Real ra[3]={0,0,0} 
          "Vector from frame_a to shape origin [m], resolved in frame_a";
        parameter Real LengthDirection[3]={1,0,0} 
          "Vector in length direction, resolved in frame_a";
        parameter Real WidthDirection[3]={0,1,0} 
          "Vector in width direction, resolved in frame_a";
        parameter Real Length=1 "Length of shape [m]";
        parameter Real Width=0.1 "Width of shape [m]";
        parameter Real Height=0.1 "Height of shape [m]";
        parameter Real Material[4]={1,0,0,0.5} "Color and specular coefficient";
        parameter Real Extra=0.0 "Additional parameter for cone and pipe";
        annotation (
          preferedView="info",
          defaultComponentName="shape",
          Coordsys(
            extent=[-100, -100; 100, 100],
            grid=[2, 2],
            component=[20, 20]),
          Icon(
            Polygon(points=[-90, 34; -90, -60; 0, -40; 60, -60; 60, 34; 0, 12; -90,
                   34], style(
                color=7,
                fillColor=46,
                fillPattern=1)),
            Polygon(points=[-88, 34; -54, 46; 10, 30; 84, 56; 60, 32; 0, 12; -88,
                  34], style(
                color=7,
                fillColor=9,
                fillPattern=1)),
            Polygon(points=[82, 54; 60, 32; 60, -60; 82, -32; 82, 54], style(
                color=7,
                fillColor=9,
                fillPattern=1)),
            Text(extent=[-100, 120; 100, 60], string="%name"),
            Text(
              extent=[-100, -61; 100, -99],
              string="%Shape",
              style(color=0))),
          Window(
            x=0.64,
            y=0.33,
            width=0.28,
            height=0.42),
          Documentation(info="A variety of shapes can be defined for animation.
Parameter vectors are defined with respect to frame_a in [m].
Parameters:
  Shape        : Type of visual shape. Predefined shapes are
                   Shape = \"box\" 
                           \"sphere\" 
                           \"cylinder\" 
                           \"cone\" 
                           \"pipe\" 
                           \"beam\"
                           \"gearwheel\"
                           \"wirebox\" 
                 External shapes are specified as DXF-files (only 3DFace is supported).
                 External shapes must be named \"1\", \"2\"  etc.. The corresponding definitions 
                 should be in files \"1.dxf\", \"2.dxf\" etc.Since the DXF-files contain color
                 and dimensions for the individual faces, the corresponding information
                 in the model is currently ignored, but the specular coefficient is 
                 utilized. The DXF-files must be found in the current directory. 
  r0[3]        : Position vector from the origin of frame_a to the origin
                 of the visual object.
  LengthDirection[3]: Unit vector in direction of length (will be normalized).
  WidthDirection[3] : Unit vector in direction of width (will be normalized).
  Length       : Length of visual shape in [m].
  Width        : Width of visual shape in [m].
  Height       : Height of visual shape in direction of a vector which is
                 orthogonal to  LengthDirection  and  WidthDirection .
  Material[4]  : = {r, g, b, specular} specifies the color of the shape.
                 [r,g,b] affects the color of diffuse and ambient reflected
                 light. Specular is a coefficient defining white specular
                 reflection. What color that is reflected also depends on the
                 color of the light sources. Note, r g, b and specular are
                 given in the range 0 .. 1. Specular=1 gives a metallic appearance.
  Extra        : Additional parameter for cone and pipe.
                 \"pipe\"     : A hollow cylinder. Extra = InnerRadius/Radius of the cylinder.
                 \"gearwheel\": A ring with gear wheel teeth. Extra = number of teeth.
In the class, the unit vectors in direction of the shape axes are computed
and are provided as terminal variables to be used in the calling object:
  nLength(3)   : Unit vector in direction of  AxisLength .
  nWidth (3)   : Unit vector orthogonal to  nLength .
  nHeight(3)   : Unit vector orthogonal to  nLength  and  nWidth.
"));
        TwoD2ThreeD twoD2ThreeD(
          x0=x0,
          y0=y0,
          r0=r0) annotation (extent=[-68, -30; -8, 30]);
        Modelica.Mechanics.MultiBody.Visualizers.FixedShape vis(
          shapeType=
                Shape,
          r_shape=
             ra,
          lengthDirection=LengthDirection,
          widthDirection=WidthDirection,
          length=Length,
          width=Width,
          height=Height,
          extra=Extra,
          color=integer([255,0,0,0; 0,255,0,0; 0,0,255,0]*(Material))) 
                       annotation (extent=[19, -40; 100, 40]);
      equation 
        connect(twoD2ThreeD.frame_a, frame_a) annotation (points=[-69.5, -0.3; -80.125,
               -0.3; -80.125, -1.3; -90.75, -1.3; -90.75, -1; -105, -1], style(
              color=10));
        connect(twoD2ThreeD.frame_b, vis.frame_a) 
          annotation (points=[-6.5, 0; 16.975, 0]);
      end Shape;
    end Parts;
  end PlanarMultiBody;
end BaseClasses;
