package Bodies 
  model RigidBody 
    annotation (Icon(
        Polygon(points=[-80, 30; -80, -30; -74, -40; -74, 40; -80, 30],
            style(
            color=0,
            fillColor=0,
            fillPattern=1)),
        Polygon(points=[-74, 40; -70, 42; -50, 42; -40, 40; 44, 40; 50, 42;
               70, 42; 76, 40; 80, 40; 82, 42; 86, 42; 90, 40; 90, -40; 88,
               -42; 84, -42; 80, -40; 76, -40; 70, -42; 50, -42; 44, -40; -40,
               -40; -50, -42; -70, -42; -74, -40; -74, 40], style(color=0,
              fillColor=1)),
        Polygon(points=[-46, -16; -28, -24; 32, -26; 42, -14; 42, 16; 30,
              26; -30, 24; -46, 14; -46, -16], style(
            color=0,
            fillColor=1,
            fillPattern=1)),
        Polygon(points=[-78, 28; -78, 20; -70, 20; -70, 34; -78, 28], style(
            color=0,
            fillColor=7,
            fillPattern=1)),
        Polygon(points=[-78, -20; -70, -20; -70, -36; -78, -28; -78, -20],
            style(
            color=0,
            fillColor=7,
            fillPattern=1)),
        Line(points=[46, 34; 78, 34; 78, -30; 46, -30], style(color=0,
              fillPattern=1)),
        Polygon(points=[80, 40; 80, -40; 84, -42; 88, -42; 90, -40; 90, 40;
               86, 42; 82, 42; 80, 40], style(
            color=0,
            fillColor=9,
            fillPattern=1)),
        Polygon(points=[-26, 22; -18, 12; -2, 12; -2, 22; -26, 22], style(
            color=0,
            fillColor=7,
            fillPattern=1)),
        Polygon(points=[-26, -22; -18, -10; -2, -10; -2, -22; -26, -22],
            style(color=0, fillColor=7)),
        Polygon(points=[2, 22; 2, 12; 14, 12; 24, 22; 2, 22], style(
            color=0,
            fillColor=7,
            fillPattern=1)),
        Polygon(points=[2, -10; 2, -22; 26, -22; 14, -10; 2, -10], style(
            color=0,
            fillColor=7,
            fillPattern=1)),
        Polygon(points=[20, 10; 28, 18; 32, 20; 32, -20; 28, -18; 20, -8;
              20, 10], style(
            color=0,
            fillColor=7,
            fillPattern=1)),
        Polygon(points=[-44, 12; -36, 18; -28, 18; -26, 10; -26, -10; -28,
              -18; -36, -18; -44, -14; -44, 12], style(
            color=0,
            fillColor=7,
            fillPattern=1)),
        Line(points=[-60, 52; -60, 42], style(
            color=0,
            fillColor=1,
            fillPattern=1)),
        Line(points=[-60, -42; -60, -52], style(
            color=0,
            fillColor=1,
            fillPattern=1)),
        Line(points=[60, 52; 60, 42], style(
            color=0,
            fillColor=1,
            fillPattern=1)),
        Line(points=[60, -42; 60, -52], style(
            color=0,
            fillColor=1,
            fillPattern=1))));
    extends Chassis.Interfaces.Body;
    parameter Modelica.SIunits.Height road_to_CG 
      "Distance from the road to the vehicle center of gravity";
    parameter Modelica.SIunits.Distance front_to_CG 
      "Distance from front axle to the vehicle center of gravity";
    parameter Modelica.SIunits.Distance rear_to_CG 
      "Distance from front axle to the vehicle center of gravity";
    parameter Modelica.SIunits.Mass vehicle_mass "Mass of the vehicle";
  protected 
    Modelica.SIunits.Force gf_vertical "Vertical forces due to gravity";
    Modelica.SIunits.Force gf_horizontal "Horizontal forces due to gravity";
    Modelica.SIunits.Position s "Vehicle position";
    Modelica.SIunits.Velocity v "Vehicle velocity";
  equation 
    // Gravity forces
    gf_horizontal = 0;
    gf_vertical = Modelica.Constants.g_n*vehicle_mass;
    
    // F=ma for longitudinal motion
    v = der(s);
    vehicle_mass*der(v) = right_front.acc_force + left_front.acc_force +
      right_rear.acc_force + left_rear.acc_force + vehicle.f;
    
    // Weight distribution
    right_front.weight = -.25*gf_vertical;
    left_front.weight = -.25*gf_vertical;
    right_rear.weight = -.25*gf_vertical;
    left_rear.weight = -.25*gf_vertical;
    
    // Vehicle position information
    vehicle.s = s;
    right_front.vehicle_position = s;
    left_front.vehicle_position = s;
    right_rear.vehicle_position = s;
    left_rear.vehicle_position = s;
    
  end RigidBody;
end Bodies;
