model RigidChassis "A lumped chassis with slipping tires" 
  extends VehicleSystems.Interfaces.Subsystems.Chassis(
                                            redeclare connector DriverBus = 
        LocalDriverBus);
  parameter Modelica.SIunits.Inertia J_wheel=1.4 "individual wheel inertia";
  Chassis.Bodies.RigidBody body(
    road_to_CG=road_to_CG,
    front_to_CG=front_to_CG,
    rear_to_CG=rear_to_CG,
    vehicle_mass=vehicle_mass) annotation (extent=[-40, -20; 0, 20]);
  Chassis.Tires.PacejkaTire right_front_tire(
    scale_factor=1,
    slip_data=slip_data,
    tire_radius=tire_radius) 
    annotation (extent=[-70,30; -50,50],   rotation=270);
  Chassis.Tires.PacejkaTire right_rear_tire(
    scale_factor=1,
    tire_radius=tire_radius,
    slip_data=slip_data) annotation (extent=[10,30; 30,50],   rotation=180);
  Chassis.Tires.PacejkaTire left_front_tire(
    scale_factor=1,
    tire_radius=tire_radius,
    slip_data=slip_data) 
    annotation (extent=[-68,-50; -48,-30],   rotation=0);
  Chassis.Tires.PacejkaTire left_rear_tire(
    scale_factor=1,
    tire_radius=tire_radius,
    slip_data=slip_data) annotation (extent=[10,-50; 30,-30],   rotation=90);
  parameter Tires.RoadSurfaces.PacejkaCoefficients slip_data=
      Tires.RoadSurfaces.dry_asphalt;
  parameter Modelica.SIunits.Radius tire_radius=0.34 "Tire radius";
  parameter Modelica.SIunits.Mass vehicle_mass=1200 "Vehicle mass";
  parameter Modelica.SIunits.Height road_to_CG=0.2 
    "Distance from road to center of gravity";
  parameter Modelica.SIunits.Distance front_to_CG=1.5 
    "Distance from front to center of gravity";
  parameter Modelica.SIunits.Distance rear_to_CG=2.5 
    "Distance from rear to center of gravity";
protected 
  connector LocalDriverBus 
    extends StandardDriverBus;
    Modelica.Blocks.Interfaces.RealSignal vehicle_speed(
                                                      redeclare type SignalType
        = Types.KPH);
  end LocalDriverBus;
public 
  VehicleSpeedSensor speed_sensor(tire_radius=tire_radius) 
    annotation (extent=[54,64; 66,76], rotation=90);
equation 
  connect(right_front_tire.motion, body.right_front) 
    annotation (points=[-52,32; -32,32; -32,16],    style(color=58));
  connect(left_rear_tire.motion, body.left_rear) 
    annotation (points=[12,-32; -8,-32; -8,-16],    style(color=58));
  connect(left_front_tire.motion, body.left_front) 
    annotation (points=[-50,-32; -32,-32; -32,-16],    style(color=58));
  connect(body.right_rear, right_rear_tire.motion) 
    annotation (points=[-8,16; -8,32; 12,32],    style(color=58));
  annotation (Diagram, Icon(
      Rectangle(extent=[-70, 60; -50, 52], style(
          pattern=0,
          fillColor=0,
          fillPattern=1)),
      Rectangle(extent=[-70, -52; -50, -60], style(
          pattern=0,
          fillColor=0,
          fillPattern=1)),
      Rectangle(extent=[50, 60; 70, 52], style(
          pattern=0,
          fillColor=0,
          fillPattern=1)),
      Rectangle(extent=[50, -52; 70, -60], style(
          pattern=0,
          fillColor=0,
          fillPattern=1)),
      Polygon(points=[-80, 30; -80, -30; -74, -40; -74, 40; -80, 30], style(
          color=0,
          fillColor=0,
          fillPattern=1)),
      Polygon(points=[-74, 40; -70, 42; -50, 42; -40, 40; 44, 40; 50, 42;
            70, 42; 76, 40; 80, 40; 82, 42; 86, 42; 90, 40; 90, -40; 88, -42;
             84, -42; 80, -40; 76, -40; 70, -42; 50, -42; 44, -40; -40, -40;
             -50, -42; -70, -42; -74, -40; -74, 40], style(color=0,
            fillColor=1)),
      Polygon(points=[-46, -16; -28, -24; 32, -26; 42, -14; 42, 16; 30, 26;
             -30, 24; -46, 14; -46, -16], style(
          color=0,
          fillColor=1,
          fillPattern=1)),
      Polygon(points=[-78, 28; -78, 20; -70, 20; -70, 34; -78, 28], style(
          color=0,
          fillColor=7,
          fillPattern=1)),
      Polygon(points=[-78, -20; -70, -20; -70, -36; -78, -28; -78, -20],
          style(
          color=0,
          fillColor=7,
          fillPattern=1)),
      Line(points=[46, 34; 78, 34; 78, -30; 46, -30], style(color=0,
            fillPattern=1)),
      Polygon(points=[80, 40; 80, -40; 84, -42; 88, -42; 90, -40; 90, 40;
            86, 42; 82, 42; 80, 40], style(
          color=0,
          fillColor=9,
          fillPattern=1)),
      Polygon(points=[-26, 22; -18, 12; -2, 12; -2, 22; -26, 22], style(
          color=0,
          fillColor=7,
          fillPattern=1)),
      Polygon(points=[-26, -22; -18, -10; -2, -10; -2, -22; -26, -22],
          style(color=0, fillColor=7)),
      Polygon(points=[2, 22; 2, 12; 14, 12; 24, 22; 2, 22], style(
          color=0,
          fillColor=7,
          fillPattern=1)),
      Polygon(points=[2, -10; 2, -22; 26, -22; 14, -10; 2, -10], style(
          color=0,
          fillColor=7,
          fillPattern=1)),
      Polygon(points=[20, 10; 28, 18; 32, 20; 32, -20; 28, -18; 20, -8; 20,
             10], style(
          color=0,
          fillColor=7,
          fillPattern=1)),
      Polygon(points=[-44, 12; -36, 18; -28, 18; -26, 10; -26, -10; -28, -18;
             -36, -18; -44, -14; -44, 12], style(
          color=0,
          fillColor=7,
          fillPattern=1)),
      Line(points=[-60, 52; -60, 42], style(
          color=0,
          fillColor=1,
          fillPattern=1)),
      Line(points=[-60, -42; -60, -52], style(
          color=0,
          fillColor=1,
          fillPattern=1)),
      Line(points=[60, 52; 60, 42], style(
          color=0,
          fillColor=1,
          fillPattern=1)),
      Line(points=[60, -42; 60, -52], style(
          color=0,
          fillColor=1,
          fillPattern=1))));
  driver_bus.steering_wheel.tau = 0;
  connect(left_front_tire.hub, wheels.left_front_wheel) annotation (points=[-58,
        -40; -100,-40; -100,0], style(color=0, rgbcolor={0,0,0}));
  connect(left_rear_tire.hub, wheels.left_rear_wheel) annotation (points=[20,
        -40; 20,-60; -100,-60; -100,0], style(color=0, rgbcolor={0,0,0}));
  connect(right_front_tire.hub, wheels.right_front_wheel) annotation (points=[
        -60,40; -100,40; -100,0], style(color=0, rgbcolor={0,0,0}));
  connect(right_rear_tire.hub, wheels.right_rear_wheel) annotation (points=[20,
        40; 20,60; -100,60; -100,0], style(color=0, rgbcolor={0,0,0}));
  connect(right_rear_tire.hub, speed_sensor.hub) annotation (points=[20,40;
        60,40; 60,64], style(
      color=0,
      rgbcolor={0,0,0},
      fillColor=10,
      rgbfillColor={135,135,135},
      fillPattern=1));
  connect(speed_sensor.v, driver_bus.vehicle_speed) annotation (points=[60,76;
        60,92], style(
      color=0,
      rgbcolor={0,0,0},
      fillColor=10,
      rgbfillColor={135,135,135},
      fillPattern=1));
end RigidChassis;
