model VehicleSpeedSensor 
  parameter Modelica.SIunits.Radius tire_radius;
  Modelica.Blocks.Math.Gain conversion(k=tire_radius) 
    annotation (extent=[-28,-10; -8,10]);
  Modelica.Mechanics.Rotational.Sensors.SpeedSensor speed_sensor 
    annotation (extent=[-70,-10; -50,10]);
  annotation (Diagram, Icon(
      Ellipse(extent=[-70,70; 70,-70],   style(color=0, fillColor=7)),
      Line(points=[0,70; 0,40],   style(color=0)),
      Line(points=[22.9,32.8; 40.2,57.3],   style(color=0)),
      Line(points=[-22.9,32.8; -40.2,57.3],   style(color=0)),
      Line(points=[37.6,13.7; 65.8,23.9],   style(color=0)),
      Line(points=[-37.6,13.7; -65.8,23.9],   style(color=0)),
      Line(points=[0,0; 9.02,28.6],   style(color=0)),
      Polygon(points=[-0.48,31.6; 18,26; 18,57.2; -0.48,31.6],     style(
          color=0,
          fillColor=0,
          fillPattern=1)),
      Ellipse(extent=[-5,5; 5,-5],   style(
          color=0,
          gradient=0,
          fillColor=0,
          fillPattern=1)),
      Line(points=[-70,0; -90,0],   style(color=0)),
      Line(points=[70,0; 100,0]),
      Text(extent=[0,70; 0,110],   string="%name")));
  Modelica.Mechanics.Rotational.Interfaces.Flange_a hub 
    annotation (extent=[-110,-10; -90,10]);
  Modelica.Blocks.Interfaces.RealSignal v(
                                        redeclare type SignalType = 
        Types.KPH) annotation (extent=[90,-10; 110,10]);
  Modelica.Blocks.Interfaces.Adaptors.SendReal speed_kph 
    annotation (extent=[50,-10; 70,10]);
  Modelica.Blocks.Math.Gain to_kph(k=3.6) 
    annotation (extent=[12,-10; 32,10]);
equation 
  connect(speed_sensor.w,conversion.u) 
    annotation (points=[-49,0; -30,0],style(color=3, rgbcolor={0,0,255}));
  connect(speed_sensor.flange_a, hub) 
    annotation (points=[-70,0; -100,0], style(color=0, rgbcolor={0,0,0}));
  connect(speed_kph.toBus, v) 
    annotation (points=[71,0; 100,0], style(color=77, rgbcolor={127,0,255}));
  connect(to_kph.u,conversion.y) 
    annotation (points=[10,0; -7,0], style(color=3, rgbcolor={0,0,255}));
  connect(to_kph.y,speed_kph.u) 
    annotation (points=[33,0; 48,0], style(color=3, rgbcolor={0,0,255}));
end VehicleSpeedSensor;
