model FuelEconomy "Compute fuel consumption" 
  
  parameter Types.TorqueLimits torque_curves annotation(choicesAllMatching=true);
  extends BaseClasses.BaseEngine;
  output Modelica.SIunits.Mass fuel_consumption;
protected 
  final parameter Real torque_map[size(torque_curves.omega,1),3]=
    [torque_curves.omega, torque_curves.min_torque, torque_curves.max_torque];
  Real min_torque;
  Real max_torque;
  Modelica.Blocks.Tables.CombiTable1D engine_map(table=torque_map, columns=
        integer(({2,3}))) 
    annotation (extent=[-10,-10; 10,10]);
public 
  Modelica.Blocks.Tables.CombiTable2D fuel_map(table=[0.0,-10,0,50,100,150,200; 
        0,0.0,0.0,0.0,0,0,0; 500,0.0,0.1,0.4,0.7,0.9,1.2; 1000,0.0,0.3,0.6,0.9,
        1.2,1.5; 2000,0.0,0.5,1.2,1.75,2.4,3.0; 3000,0.0,0.8,1.75,2.7,3.7,4.6; 
        4000,0.0,1.3,2.5,3.75,5.0,6.25; 5000,0.0,1.7,3.4,5.0,6.5,8.1; 6000,0.0,
        2.7,4.5,6.4,8.3,10.2])               annotation (extent=[-10,40; 10,60]);
equation 
  engine_map.u[1]             = max(omega,0.01);
  engine_map.u[2]             = max(omega,0.01);
  fuel_map.u1 = omega;
  fuel_map.u2 = tau;
  der(fuel_consumption) = 1e-2*fuel_map.y;
  min_torque =engine_map.y[1];
  max_torque =engine_map.y[2];
  tau = min(1.0,driver_bus.accelerator_pedal.s)*(max_torque-min_torque)+min_torque;
end FuelEconomy;
