package Components 
  model SteadyStateTorqueConverter 
    extends Modelica.Mechanics.Rotational.Interfaces.Compliant;
    parameter Real speed_ratios[:];
    parameter Real torque_ratios[:];
    parameter Real K_values[:];
  protected 
    parameter Real table[:,3] = [speed_ratios, torque_ratios, K_values];
    Real speed_ratio;
    Real torque_ratio;
    Real K;
    Modelica.SIunits.AngularVelocity w_turbine;
    Modelica.SIunits.AngularVelocity w_pump;
    Modelica.SIunits.Torque tau_turbine;
    Modelica.SIunits.Torque tau_pump;
  public 
    Modelica.Blocks.Tables.CombiTable1Ds torque_converter_table 
      annotation (extent=[-10,-10; 10,10]);
  equation 
    // Alias definitions
    w_turbine = der(flange_a.phi);
    w_pump = der(flange_b.phi);
    tau_turbine = flange_a.tau;
    tau_pump = flange_b.tau;
    // Basic torque converter relationships
    speed_ratio = w_turbine/max(0.01,w_pump);
    tau_pump = (K/w_pump)^2;
    // Tabular relationships
    torque_converter_table.u = speed_ratio;
    torque_ratio = torque_converter_table.y[1];
    K = torque_converter_table.y[2];
  end SteadyStateTorqueConverter;
end Components;
