partial model ConventionalVehicle 
  "Configuration of subsystems associated with a conventional vehicle" 
  
  replaceable VehicleSystems.Documentation.Driver driver 
                                              extends 
    VehicleSystems.Interfaces.Subsystems.Driver      annotation (
    choicesAllMatching=true,
    dialog(tab="General", group="Conditions"),
    extent=[-100,70; -80,90]);
  
  replaceable VehicleSystems.Documentation.Powerplant powerplant 
     extends VehicleSystems.Interfaces.Subsystems.Powerplant 
    annotation (
    choicesAllMatching=true,
    dialog(group="PlantModels", tab="General"),
    extent=[-68, -30; -48, -10]);
  
  replaceable VehicleSystems.Documentation.Electrical electrical 
     extends VehicleSystems.Interfaces.Subsystems.Electrical 
    annotation (
    choicesAllMatching=true,
    dialog(group="PlantModels", tab="General"),
    extent=[-90, -90; -70, -70]);
  
  replaceable VehicleSystems.Documentation.Transmission transmission 
     extends VehicleSystems.Interfaces.Subsystems.Transmission 
    annotation (
    choicesAllMatching=true,
    dialog(group="PlantModels", tab="General"),
    extent=[-30, -30; -10, -10]);
  
  replaceable VehicleSystems.Documentation.Accessories accessories 
     extends VehicleSystems.Interfaces.Subsystems.Accessories 
    annotation (
    choicesAllMatching=true,
    dialog(group="PlantModels", tab="General"),
    extent=[-108, -30; -88, -10]);
  
  replaceable VehicleSystems.Documentation.Chassis chassis 
                                                extends 
    VehicleSystems.Interfaces.Subsystems.Chassis 
                                      annotation (
    choicesAllMatching=true,
    dialog(group="PlantModels", tab="General"),
    extent=[92, -30; 112, -10]);
  
  replaceable VehicleSystems.Documentation.Driveline driveline 
                                                    extends 
    VehicleSystems.Interfaces.Subsystems.Driveline 
                                        annotation (
    choicesAllMatching=true,
    dialog(group="PlantModels", tab="General"),
    extent=[12, -30; 32, -10]);
  
  replaceable VehicleSystems.Documentation.Brakes brakes 
                                              extends 
    VehicleSystems.Interfaces.Subsystems.Brakes 
                                     annotation (
    choicesAllMatching=true,
    dialog(group="PlantModels", tab="General"),
    extent=[50, -30; 70, -10]);
  
  inner replaceable VehicleSystems.Documentation.Environment environment 
                extends VehicleSystems.Interfaces.Subsystems.Environment 
                                          annotation (
    choicesAllMatching=true,
    dialog(tab="General", group="Conditions"),
    extent=[90, -90; 110, -70]);
  
  replaceable VehicleSystems.Documentation.PowerplantController 
    powerplant_control extends 
    VehicleSystems.Interfaces.Subsystems.PowerplantController 
    annotation (
    choicesAllMatching=true,
    dialog(group="Controllers", tab="General"),
    extent=[-70, 10; -50, 30]);
  
  replaceable VehicleSystems.Documentation.TransmissionController 
    transmission_control extends 
    VehicleSystems.Interfaces.Subsystems.TransmissionController 
    annotation (
    choicesAllMatching=true,
    dialog(group="Controllers", tab="General"),
    extent=[-30, 10; -10, 30]);
  
  replaceable VehicleSystems.Documentation.DrivelineController 
    driveline_control extends 
    VehicleSystems.Interfaces.Subsystems.DrivelineController 
    annotation (
    choicesAllMatching=true,
    dialog(group="Controllers", tab="General"),
    extent=[10, 10; 30, 30]);
  
  replaceable VehicleSystems.Documentation.BrakesController brake_control 
     extends VehicleSystems.Interfaces.Subsystems.BrakesController 
    annotation (
    choicesAllMatching=true,
    dialog(group="Controllers", tab="General"),
    extent=[50, 10; 70, 30]);
  
  replaceable VehicleSystems.Documentation.AccessoriesController 
    accessory_control extends 
    VehicleSystems.Interfaces.Subsystems.AccessoriesController 
    annotation (
    choicesAllMatching=true,
    dialog(group="Controllers", tab="General"),
    extent=[-110, 10; -90, 30]);
  
  replaceable VehicleSystems.Documentation.ElectricalController 
    electrical_control extends 
    VehicleSystems.Interfaces.Subsystems.ElectricalController 
    annotation (
    choicesAllMatching=true,
    dialog(group="Controllers", tab="General"),
    extent=[-50, -70; -30, -50]);
  
  replaceable VehicleSystems.Documentation.ChassisController chassis_control 
                    extends 
    VehicleSystems.Interfaces.Subsystems.ChassisController 
                                                annotation (
    choicesAllMatching=true,
    dialog(group="Controllers", tab="General"),
    extent=[90, 10; 110, 30]);
  
  replaceable VehicleSystems.Documentation.VehicleController vehicle_control 
                    extends 
    VehicleSystems.Interfaces.Subsystems.VehicleController 
    annotation (
    choicesAllMatching=true,
    dialog(group="Controllers", tab="General"),
    extent=[-10, 50; 10, 70]);
equation 
  
  connect(electrical.power, powerplant.power) annotation (points=[-80,-70; -80,
        -46; -68,-46; -68,-30],              style(
      color=1,
      fillColor=7,
      fillPattern=1));
  connect(powerplant.crankshaft, transmission.input_shaft) annotation (points=[
        -48, -20; -30, -20], style(
      color=0,
      fillColor=7,
      fillPattern=1));
  connect(accessories.drive, powerplant.accessory_drive) annotation (points=[-88,
         -20; -68, -20], style(
      color=0,
      fillColor=7,
      fillPattern=1));
  connect(accessories.power, electrical.power) annotation (points=[-104, -30; -104,
         -46; -80, -46; -80, -70], style(
      color=1,
      fillColor=7,
      fillPattern=1));
  connect(electrical.power, chassis.power) annotation (points=[-80, -70; -80, -46;
         112, -46; 112, -30], style(
      color=1,
      gradient=1,
      fillColor=8));
  connect(transmission.output_shaft, driveline.driveshaft) annotation (points=[
        -10, -20; 12, -20], style(
      color=0,
      fillColor=8,
      fillPattern=1));
  connect(transmission.power, electrical.power) annotation (points=[-10, -30; -10,
         -46; -80, -46; -80, -70], style(
      color=1,
      fillColor=7,
      fillPattern=1));
  connect(driveline.power, electrical.power) annotation (points=[30, -30; 30, -46;
         -80, -46; -80, -70], style(
      color=1,
      fillColor=7,
      fillPattern=1));
  connect(brakes.power, electrical.power) 
    annotation (points=[54,-30; 54,-46; -80,-46; -80,-70],     style(color=1));
  connect(powerplant_control.powerplant_bus, powerplant.powerplant_bus) 
    annotation (points=[-60, 10; -60, -10], style(
      color=0,
      pattern=3,
      thickness=2));
  connect(driveline_control.driveline_bus, driveline.driveline_bus) annotation (
     points=[20, 10; 20, -10], style(
      color=0,
      pattern=3,
      thickness=2));
  connect(electrical.electrical_bus, electrical_control.electrical_bus) 
    annotation (points=[-70, -80; -40, -80; -40, -70], style(
      color=0,
      pattern=3,
      thickness=2));
  connect(brake_control.brakes_bus, brakes.brakes_bus) annotation (points=[60,
        10; 60, -10; 60.2, -10], style(
      color=0,
      pattern=3,
      thickness=2));
  connect(transmission_control.transmission_bus, transmission.transmission_bus) 
     annotation (points=[-20,10; -20,-10; -20,-10],      style(
      color=0,
      pattern=3,
      thickness=2));
  connect(accessory_control.accessories_bus, accessories.accessories_bus) 
    annotation (points=[-100, 10; -100, -10], style(
      color=0,
      pattern=3,
      thickness=2));
  connect(chassis_control.chassis_bus, chassis.chassis_bus) annotation (points=
        [100, 10; 100, -10], style(
      color=0,
      pattern=3,
      thickness=2));
  annotation (Diagram, Coordsys(extent=[-120, -100; 120, 100]));
  connect(brakes.wheels_a, driveline.wheels) 
    annotation (points=[50, -20; 32, -20]);
  connect(brakes.wheels_b, chassis.wheels) 
    annotation (points=[70, -20; 92, -20]);
  connect(driveline.driveline_mounts, chassis.driveline_mounts) 
    annotation (points=[22,-30; 22,-36; 100,-36; 100,-30],   style(color=0));
  connect(transmission.transmission_mounts, chassis.transmission_mounts) 
    annotation (points=[-20,-30; -20,-38; 104,-38; 104,-30],     style(color=0));
  connect(powerplant.powerplant_mounts, chassis.powerplant_mounts) annotation (
      points=[-58,-30; -58,-40; 108,-40; 108,-30],     style(color=0));
  connect(vehicle_control.global_bus, accessory_control.global_bus) 
    annotation (points=[0, 50; 0, 40; -100, 40; -100, 30], style(color=0));
  connect(vehicle_control.global_bus, powerplant_control.global_bus) 
    annotation (points=[0, 50; 0, 40; -60, 40; -60, 30], style(color=0));
  connect(vehicle_control.global_bus, transmission_control.global_bus) 
    annotation (points=[0, 50; 0, 40; -20, 40; -20, 30], style(color=0));
  connect(vehicle_control.global_bus, driveline_control.global_bus) 
    annotation (points=[0, 50; 0, 40; 20, 40; 20, 30], style(color=0));
  connect(vehicle_control.global_bus, brake_control.global_bus) 
    annotation (points=[0, 50; 0, 40; 60, 40; 60, 30], style(color=0));
  connect(vehicle_control.global_bus, chassis_control.global_bus) 
    annotation (points=[0, 50; 0, 40; 100, 40; 100, 30], style(color=0));
  connect(vehicle_control.global_bus, electrical_control.global_bus) 
    annotation (points=[0, 50; 0, 40; -40, 40; -40, -50], style(color=0));
  connect(brakes.brake_mounts, chassis.brake_mounts) annotation (points=[60,-30;
        60,-34; 96,-34; 96,-30], style(color=10, rgbcolor={95,95,95}));
  connect(driver.driver_bus, powerplant.driver_bus) annotation (points=[-80,80;
        -44,80; -44,-4; -52,-4; -52,-10], style(
      color=0,
      rgbcolor={0,0,0},
      pattern=4));
  connect(driver.driver_bus, transmission.driver_bus) annotation (points=[-80,
        80; -36,80; -36,-4; -26,-4; -26,-10], style(
      color=0,
      rgbcolor={0,0,0},
      pattern=4));
  connect(driver.driver_bus, brakes.driver_bus) annotation (points=[-80,80; 44,
        80; 44,-4; 54,-4; 54,-10], style(
      color=0,
      rgbcolor={0,0,0},
      pattern=4));
  connect(driver.driver_bus, chassis.driver_bus) annotation (points=[-80,80;
        116,80; 116,-4; 108,-4; 108,-10], style(
      color=0,
      rgbcolor={0,0,0},
      pattern=4));
  connect(driver.driver_bus, electrical_control.driver_bus) annotation (points=
        [-80,80; -74,80; -74,-60; -50,-60], style(
      color=0,
      rgbcolor={0,0,0},
      pattern=4));
  connect(driver.driver_bus, powerplant_control.driver_bus) annotation (points=
        [-80,80; -44,80; -44,24; -50,24], style(
      color=0,
      rgbcolor={0,0,0},
      pattern=4));
  connect(driver.driver_bus, transmission_control.driver_bus) annotation (
      points=[-80,80; -36,80; -36,24; -30,24], style(
      color=0,
      rgbcolor={0,0,0},
      pattern=4));
end ConventionalVehicle;
