partial model Brakes "Model of the physical brake interactions at each wheel" 
  
  extends VehicleSystems.Icons.Brakes;
  extends VehicleSystems.Interfaces.Subsystems.Brakes;
  annotation (
    Diagram(
      Rectangle(extent=[-20, 60; 0, -60], style(
          color=0,
          gradient=2,
          fillColor=10)),
      Ellipse(extent=[-20, 60; 20, -60], style(
          color=0,
          gradient=3,
          fillColor=10)),
      Ellipse(extent=[-40, 60; 0, -60], style(
          color=0,
          gradient=3,
          fillColor=10)),
      Polygon(points=[-34, 24; -30, 14; -24, 20; -20, 18; -14, 2; -18, -12; -10,
             -22; -4, 6; -12, 24; -18, 34; -28, 32; -34, 24], style(
          color=0,
          gradient=2,
          fillColor=10)),
      Polygon(points=[-38, 24; -34, 14; -28, 20; -24, 18; -18, 2; -22, -12; -14,
             -22; -8, 6; -16, 24; -22, 34; -32, 32; -38, 24], style(
          color=0,
          gradient=2,
          fillColor=10)),
      Ellipse(extent=[-34, 26; -30, 22], style(
          color=0,
          gradient=2,
          fillColor=10)),
      Ellipse(extent=[-24, 26; -20, 22], style(
          color=0,
          gradient=2,
          fillColor=10)),
      Ellipse(extent=[-14, 2; -10, 6], style(
          color=0,
          gradient=2,
          fillColor=10)),
      Ellipse(extent=[-18, -14; -14, -10], style(
          color=0,
          gradient=2,
          fillColor=10)),
      Text(
        extent=[-100, 80; 20, 60],
        string="Brake pedal connection",
        style(color=58, arrow=1)),
      Line(points=[-60, 76; -60, 80; -60, 90], style(color=58, arrow=1)),
      Text(
        extent=[-82, -64; 44, -82],
        string="Individual wheel torques",
        style(color=10, arrow=1)),
      Line(points=[-40, -60; -86, -4], style(color=10, arrow=1)),
      Text(
        extent=[-102, -90; -26, -110],
        string="Electrical supply",
        style(color=1, arrow=1)),
      Line(points=[-26, -100; -20, -100; -10, -100], style(color=1, arrow=1)),
      Line(points=[20, -60; 86, -4], style(color=10, arrow=1))),
    Documentation(info="<html>
<p>
For the purposes of use in an expandable modular vehicle model, the Brakes subsystem is defined as a subsystem that transfers torque to the road wheels found in the Chassis subsystem.  The magnitude of torque transferred will depend on Driver input.
</p>
<p>
The Brakes subsystem has the capacity to accept electrical control signals for the purpose of modifying its internal states, whilst also producing electrical signals of its own for use by other subsystems.  The Controller Bus carries these signals.  The Brakes subsystem also has an electrical current flow associated with it for energy exchange with the Electrical subsystem.
</p>
<p>
It is sufficient to consider the Brakes as a black box subsystem with a controller for connection to the Controller Bus, and capability to supply four individual wheel torques to the Chassis subsystem as shown in Figure 6.9.1.  The subsystem also includes an electrical connection to the Electrical subsystem.
</p>
<p>
The Brakes subsystem also has a direct connection to the Driver through the brake pedal.  It is assumed that brake pedal actuator and sensor models will reside in the Brakes subsystem.  For vehicles with a vehicle system controller that interprets the Driver braking demand, the sensed braking demand signal would be generated and placed on the Control Bus from within the Brakes subsystem. 
</p>
</html>
"), Icon);
  
end Brakes;
