model Brakes "Simple proportional (constant gain) braking model" 
  parameter Modelica.SIunits.Torque max_torque=1000 
    "Maximum combined brake torque";
  extends Interfaces.Subsystems.Brakes;
  Modelica.Blocks.Math.Gain compute_torque(k=max_torque/4.0) 
    annotation (extent=[-10,50; 10,70], rotation=270);
  annotation (Diagram);
  Modelica.Mechanics.Rotational.Fixed ground 
    annotation (extent=[-90,-40; -70,-20], rotation=270);
  Modelica.Mechanics.Rotational.Brake right_front(mue_pos=[0,1]) 
    annotation (extent=[-60,20; -40,40], rotation=270);
  Modelica.Mechanics.Rotational.Brake left_front(mue_pos=[0,1]) 
    annotation (extent=[-60,-40; -40,-20], rotation=270);
  Modelica.Mechanics.Rotational.Brake right_rear(mue_pos=[0,1]) 
    annotation (extent=[40,20; 60,40], rotation=90);
  Modelica.Mechanics.Rotational.Brake left_rear(mue_pos=[0,1]) 
    annotation (extent=[40,-40; 60,-20], rotation=90);
equation 
  connect(pedal_position.s,compute_torque.u)             annotation (points=[
        -19,80; 2.20429e-015,80; 2.20429e-015,72], style(color=3, rgbcolor={0,0,
          255}));
  connect(wheels_a, wheels_b) annotation (points=[-100,0; 100,0]);
  connect(right_front.flange_b, wheels_a.right_front_wheel) annotation (points=
        [-50,20; -50,0; -100,0], style(color=0, rgbcolor={0,0,0}));
  connect(left_front.flange_a, wheels_a.left_front_wheel) annotation (points=[
        -50,-20; -50,0; -100,0], style(color=0, rgbcolor={0,0,0}));
  connect(left_front.bearing, ground.flange_b) annotation (points=[-60,-30; -70,
        -30; -70,-30; -80,-30], style(color=0, rgbcolor={0,0,0}));
  connect(right_front.bearing, ground.flange_b) annotation (points=[-60,30; -80,
        30; -80,-30], style(color=0, rgbcolor={0,0,0}));
  connect(compute_torque.y,right_front.f_normalized)  annotation (points=[
        -2.0206e-015,49; -2.0206e-015,30; -39,30], style(color=3, rgbcolor={0,0,
          255}));
  connect(left_front.f_normalized,compute_torque.y)  annotation (points=[-39,
        -30; -2.0206e-015,-30; -2.0206e-015,49], style(color=3, rgbcolor={0,0,
          255}));
  connect(right_rear.f_normalized,compute_torque.y)  annotation (points=[39,30; 
        -2.0206e-015,30; -2.0206e-015,49], style(color=3, rgbcolor={0,0,255}));
  connect(left_rear.f_normalized,compute_torque.y)  annotation (points=[39,-30; 
        -2.0206e-015,-30; -2.0206e-015,49], style(color=3, rgbcolor={0,0,255}));
  connect(left_rear.bearing, right_rear.bearing) annotation (points=[60,-30; 80,
        -30; 80,30; 60,30], style(color=0, rgbcolor={0,0,0}));
  connect(left_rear.bearing, ground.flange_b) annotation (points=[60,-30; 80,
        -30; 80,-60; -80,-60; -80,-30], style(color=0, rgbcolor={0,0,0}));
  connect(right_rear.flange_a, wheels_b.right_rear_wheel) 
    annotation (points=[50,20; 50,0; 98,0], style(color=0, rgbcolor={0,0,0}));
  connect(left_rear.flange_b, wheels_b.left_rear_wheel) 
    annotation (points=[50,-20; 50,0; 98,0], style(color=0, rgbcolor={0,0,0}));
end Brakes;
