model Chassis "No-slip, lumped chassis" 
  parameter Modelica.SIunits.Mass vehicle_mass=1200;
  parameter Modelica.SIunits.Radius tire_radius=0.34;
  extends VehicleSystems.Icons.Chassis;
  extends VehicleSystems.Interfaces.Subsystems.Chassis;
  annotation (Diagram);
  Modelica.Mechanics.Rotational.Inertia effective_inertia(J=vehicle_mass*
        tire_radius^2) annotation (extent=[-40, -10; -20, 10]);
  Modelica.Mechanics.Rotational.Damper load(d=1) 
    annotation (extent=[10, -30; 30, -10], rotation=90);
  Modelica.Mechanics.Rotational.Fixed ground 
    annotation (extent=[10, -50; 30, -30]);
  Modelica.SIunits.Position d=tire_radius*effective_inertia.phi;
  Modelica.SIunits.Velocity v=tire_radius*effective_inertia.w;
  Modelica.SIunits.Acceleration a=der(v);
  Modelica.Mechanics.Rotational.Sensors.AngleSensor steering_angle 
    annotation (extent=[20,70; 40,90], rotation=180);
equation 
  connect(effective_inertia.flange_a, wheels.left_front_wheel) 
    annotation (points=[-40, 0; -96, 0], style(color=0));
  connect(effective_inertia.flange_a, wheels.left_rear_wheel) 
    annotation (points=[-40, 0; -96, 0], style(color=0));
  connect(effective_inertia.flange_a, wheels.right_front_wheel) 
    annotation (points=[-40, 0; -96, 0], style(color=0));
  connect(effective_inertia.flange_a, wheels.right_rear_wheel) 
    annotation (points=[-40, 0; -100, 0], style(color=0));
  connect(ground.flange_b, load.flange_a) 
    annotation (points=[20, -40; 20, -30], style(color=0));
  connect(load.flange_b, effective_inertia.flange_b) 
    annotation (points=[20, -10; 20, 0; -20, 0], style(color=0));
  connect(steering_angle.flange_a, driver_bus.steering_wheel) annotation (
      points=[40,80; 60,80; 60,94], style(color=0, rgbcolor={0,0,0}));
end Chassis;
