model Driver "Max acceleration Driver" 
  extends VehicleSystems.Icons.Driver;
  extends VehicleSystems.Interfaces.Subsystems.Driver;
  parameter Real initial_accel_request(
    min=0,
    max=1) = 1 "Initial accelerator pedal position";
  parameter Real initial_brake_request(
    min=0,
    max=1) = 0 "Initial brake pedal position";
  parameter Real final_accel_request(
    min=0,
    max=1) = 1 "Final accelerator pedal position";
  parameter Real final_brake_request(
    min=0,
    max=1) = 0 "Final brake pedal position";
  parameter Modelica.SIunits.Time accel_time=10 "Time of accel apply";
  parameter Modelica.SIunits.Time brake_time=10 "Time of brake apply";
  annotation (Diagram);
  Modelica.Blocks.Sources.Step brake_signal(
    height=final_brake_request,
    offset=initial_brake_request,
    startTime=brake_time)   annotation (extent=[10, -10; 30, 10]);
  Modelica.Blocks.Sources.Step accel_signal(
    height=final_accel_request,
    offset=initial_accel_request,
    startTime=accel_time)   annotation (extent=[10, -70; 30, -50]);
  Modelica.Mechanics.Translational.Position brake_actuator 
    annotation (extent=[50,-10; 70,10]);
  Modelica.Mechanics.Translational.Position accel_actuator 
    annotation (extent=[50,-70; 70,-50]);
  Modelica.Mechanics.Rotational.Position steering_wheel 
    annotation (extent=[50,50; 70,70]);
  Modelica.Mechanics.Rotational.Fixed ground annotation (extent=[50,30; 70,50]);
  Modelica.Blocks.Sources.Constant steering_angle(k=0) 
    annotation (extent=[12,50; 32,70]);
equation 
  connect(brake_actuator.s_ref,brake_signal.y) 
    annotation (points=[48,0; 31,0], style(color=3, rgbcolor={0,0,255}));
  connect(brake_actuator.flange_b, driver_bus.brake_pedal) 
    annotation (points=[70,0; 100,0], style(color=58, rgbcolor={0,191,0}));
  connect(accel_actuator.s_ref,accel_signal.y) 
    annotation (points=[48,-60; 31,-60], style(color=3, rgbcolor={0,0,255}));
  connect(accel_actuator.flange_b, driver_bus.accelerator_pedal) annotation (
      points=[70,-60; 80,-60; 80,0; 94,0], style(color=58, rgbcolor={0,191,0}));
  connect(steering_angle.y,steering_wheel.phi_ref) 
    annotation (points=[33,60; 48,60], style(color=3, rgbcolor={0,0,255}));
  connect(steering_wheel.bearing, ground.flange_b) 
    annotation (points=[60,50; 60,40], style(color=0, rgbcolor={0,0,0}));
  connect(steering_wheel.flange_b, driver_bus.steering_wheel) annotation (
      points=[70,60; 80,60; 80,0; 100,0], style(color=0, rgbcolor={0,0,0}));
end Driver;
