model Powerplant "Constant torque output" 
  parameter Modelica.SIunits.AngularVelocity w0;
  parameter Modelica.SIunits.Torque requested_torque=50;
  extends VehicleSystems.Icons.Powerplant;
  extends VehicleSystems.Interfaces.Subsystems.Powerplant;
  Modelica.Mechanics.Rotational.Inertia flywheel(J=J_flywheel, w(start=w0)) 
    annotation (extent=[30, -10; 50, 10]);
  Modelica.Mechanics.Rotational.Torque engine 
    annotation (extent=[-10, -40; 10, -20]);
  Modelica.Blocks.Sources.Constant torque(k=requested_torque) 
    annotation (extent=[-60, -40; -40, -20]);
  annotation (Diagram, Icon(Text(
        extent=[-100, 100; 100, 48],
        string="SimpleEngine",
        style(color=1))));
  parameter Modelica.SIunits.Inertia J_flywheel=0.08 "flywheel inertia";
  Modelica.Mechanics.Translational.Sensors.PositionSensor pedal_sensor 
    annotation (extent=[20,70; 40,90], rotation=180);
  Modelica.Mechanics.Rotational.Fixed ground 
    annotation (extent=[-10,-60; 10,-40]);
equation 
  connect(torque.y,engine.tau) 
    annotation (points=[-39, -30; -12, -30], style(color=3));
  connect(engine.flange_b, flywheel.flange_a) 
    annotation (points=[10, -30; 30, -30; 30, 0], style(color=0));
  connect(flywheel.flange_a, accessory_drive) 
    annotation (points=[30, 0; -100, 0], style(color=0));
  connect(flywheel.flange_b, crankshaft) 
    annotation (points=[50, 0; 100, 0], style(color=0));
  connect(pedal_sensor.flange_a, driver_bus.accelerator_pedal) annotation (points=[40,
        80; 60,80; 60,98], style(color=58, rgbcolor={0,191,0}));
  connect(ground.flange_b, engine.bearing) 
    annotation (points=[0,-50; 0,-40], style(color=0, rgbcolor={0,0,0}));
end Powerplant;
