package VehicleSystems "A package containing interfaces definitions for vehicle system modeling"
constant String directory=classDirectory();

annotation (Documentation(info="<html>
<p>
<h1>Introduction</h1>
The <tt>VehicleSystems</tt> package contains the <b>interface definitions</b> for Modelica models so that they fit into a common framework for vehicle system modeling.  This package is modeled (as closely as possible) after the Ford Vehicle Modeling Architecture (VMA) specification which, at least at present, defines interfaces for Simulink models used in vehicle simulations.
</p>
<p>
<h1>Motivation</h1>
There are two motivations for defining such a package.  They are, in order of importance:
<ul>
<li>To define an architecture for exchange of Modelica models for developers and users of Modelica models.  Although the Modelica standard library goes a long way in defining appropriate interfaces for specific engineering domains, it doesn't really define interfaces for complete systems.  The goal of this package is to define such interfaces for vehicle systems.</li>
<li>Another expected benefit of this approach is that it will facilitate the use of Modelica models in Simulink.  There are several technical barriers to completely interchangeable models, but by at least trying to define common interfaces we hope to minimize any \"tweaking\" that must be done in order to use these models from within Simulink</li>
</ul>
</p>
<p>
<h1>Open Issues</h1>
<ul>
<li>Inertia Cascading - there is currently no easy way to support this automatically as part of the S-function generation process</li>
<li>Bus Definitions - these will invariably be different from Simulink to Modelica.  For one thing, the naming system in Modelica capitalizes on the ability to hierarchically construct the bus.  Furthermore, the S-function generation capability of Dymola don't \"understand\" these bus definitions.  Finally, the naming is also affected by the fact that Modelica supports physical type associations that are <b>not</b> related to the name.
<li>Additional physical interfaces - these are present in these models to support applications typically done in Modelica.  For example, Modelica is well suited to NVH and shift-quality studies.  Such studies require us to include the effects of the mounts.  Ideally, we would like the framework to support full 3D motion of the powertrain although at the moment the interfaces are constrained to a simpler 1D formulation.</li>
</ul>
</html>
"), uses(Modelica(version="2.2")));
end VehicleSystems;
